library(ncdf4)
library(fields)
library(parallel)

# Init
if(0){
  nc = nc_open("S:/COSMO_C15/20020202/out01/lffd2002020118c.nc")
  FR_LAND = ncvar_get(nc,"FR_LAND")
  nc_close(nc)
  
  model = list()
  MODEL = c("C15","ERA","ERA5","AMPS","T15")
  for( im in seq(MODEL) ){
    print(MODEL[im])
    nc = nc_open(paste("T:/cosmoPaper/2dmm/paper/mm",MODEL[im],".nc",sep=""))
    model[[ MODEL[im] ]] = list()
    time = ncvar_get(nc,"time")
    if(MODEL[im]=="AMPS"){ time = strptime(as.character(floor(time)),"%Y%m%d",tz="UTC") }
    if(MODEL[im]=="C15" | MODEL[im]=="T15" ){ time = as.POSIXct(time,origin=sub("seconds since ","",ncatt_get(nc,"time")$units),tz="UTC") }
    if(MODEL[im]=="ERA" | MODEL[im]=="ERA5"){ time = as.POSIXct(time*3600,origin=sub("hours since ","",ncatt_get(nc,"time")$units),tz="UTC") }
    model[[ MODEL[im] ]] $time = as.POSIXct(strftime(time,"%Y/%m/15",tz="UTC"),tz="UTC")
    VAR = c("T_2M","FF","PS")
    for( ivar in seq(VAR) ){
      model[[ MODEL[im] ]] [[ VAR[ivar] ]] = ncvar_get(nc,VAR[ivar])
    }
    oldT = model[[ MODEL[im] ]] [[ "T_2M" ]]
    p = model[[ MODEL[im] ]] [[ "PS" ]]
    p_0 = 100000
    R_L = 287
    c_p = 1003
    model[[ MODEL[im] ]] [[ "Theta" ]] = oldT * (p_0/p)^(R_L/c_p)
    nc_close(nc)
    VAR = c(VAR,"Theta")
  }

}

# plot # review
if(1){
  PRINT=1
  for( REF in c("C15","T15") ){
    
    if(REF=="C15"){ IVAR=(1:2) }else{IVAR=1}
    IVAR=(1:2)
    for( ivar in IVAR ){
      WIDTH=6
      HEIGHT=3
      filename = list(T_2M=list(C15="fig03",T15="fig04"),FF=list(C15="fig05",T15="sup01"))[[VAR[ivar]]][[REF]]
      #if(PRINT){ pdf(file=paste("T:/cosmoPaper/figures/",filename,".pdf",sep=""),width=WIDTH,height=HEIGHT) }
      if(PRINT){ png(filename=paste("T:/cosmoPaper/figures/",filename,".png",sep=""), width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
      #if(PRINT){ png(filename=paste("T:/cosmoPaper/2dmm/paper/pics/",REF,"minusModell_",VAR[ivar],".png",sep=""), width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
      layout(cbind(matrix(rep(1:8,each=3),byrow=TRUE,ncol=4*3),rep(9,2),rep(9,2)))
      CEX=1
      par(mai=rep(0.01,4),omi=c(.05,.2,0.45,.02),cex=CEX)
      UNIT = c("K","m s^-1","hPa","K") #,"W m^-2","W m^-2","W m^-2","W m^-2")
      if(ivar%in%c(1,4)){
        BREAKS = seq(-11.5,11.5,1)
        ZLIM = range(BREAKS)
        COLORS = c(
          colorRampPalette(c("purple","blue","white","red","yellow"),space ="Lab")(length(BREAKS)-1)
        )
      } 
      if(ivar%in%c(2)){
        BREAKS = seq(-6.5,6.5,1)
        ZLIM = range(BREAKS)
        COLORS = c(
          colorRampPalette(c("blue","white","red"),space ="Lab")(length(BREAKS)-1)
        )
      } 
      firstYear=2014
      lastYear=2016
      for( yy in 1:2 ){
        for( xx in 1:4 ){
          if(yy==1){ month = c(10:12,1:3) }
          if(yy==2){ month = 4:9 }
          VS = c("AMPS","ERA5","ERA","ERA")[xx]
          XLAB = c("AMPS","ERA5","ERA-Interim","ERA-Interim")[xx]
          if(xx==4){
            REFtime = model[[ REF ]] [["time"]] %in% model[[ VS ]] [["time"]]
            VStime =  model[[ VS ]] [["time"]] %in% model[[ REF ]] [["time"]] [REFtime]
          }else{
            REFtime = model[[ REF ]] [["time"]] %in% model[[ VS ]] [["time"]] & model[[ REF ]] [["time"]] >= as.POSIXct(paste(firstYear,"/01/01",sep="")) & model[[ REF ]] [["time"]] <= as.POSIXct(paste(lastYear,"/12/31",sep=""))
            VStime =  model[[ VS ]] [["time"]] %in% model[[ REF ]] [["time"]] [REFtime]
          }
          ZZ = model[[ REF ]] [[VAR[ivar]]] [,, REFtime] - model[[ VS ]] [[VAR[ivar]]] [ ,, VStime ]
          
          mindex = as.numeric(strftime(model[[ VS ]] [["time"]]  [VStime], "%m")) %in% month
          z = apply( ZZ[,,mindex], 1:2, mean )
          #z = ZZ[,,1]*0
          if(ivar%in%c(3)) z = z/100
          # A = matrix(rep(1:16,each=2),ncol=4)
          # A = A[,rep(1:4,each=2)]
          reduce = function(data,factor){
            makeMean = function(x){
              dim(x)=c(factor,length(x)/factor)
              return(apply(x,2,mean))
            }
            return(apply(t(apply(data,1,makeMean)),2,makeMean))
          }
          # reduce(A,2)
          #if( any( z<ZLIM[1] | z>ZLIM[2] ) ){print("z out of ZLIM-range")}
          
          print(range(z,na.rm = TRUE))
          z[z<ZLIM[1]] = ZLIM[1]
          z[z>ZLIM[2]] = ZLIM[2]
          image(reduce(z,2), zlim=ZLIM,xaxt="n",yaxt="n",col=COLORS,breaks=BREAKS,asp=1)
          zz = is.na(z); zz[zz==FALSE] = NA
          if(VS=="AMPS") image(zz,xaxt="n",yaxt="n",col="gray",add=TRUE)
          #image(z*0, zlim=ZLIM,xaxt="n",yaxt="n",col=COLORS,breaks=BREAKS,asp=1)
          contour(reduce(FR_LAND,2),labels="",add=TRUE)
          box(lwd=1)
          
          if(yy==1){ mtext( XLAB ,side=3,line=0.1,font=1) }
          if(yy==1 && xx%in%1:3){ mtext( paste(firstYear,"-",lastYear,sep="") ,side=3,line=1,font=1) }
          if(yy==1 && xx==4){ mtext( "2002-2016" ,side=3,line=1,font=1) }
          if(xx==1){ 
            if(yy==1){ YLAB = "Summer" }
            if(yy==2){ YLAB = "Winter" }
            mtext( YLAB ,side=2,line=0.1,font=1) 
          }
          
        }#VS
      }#month
      breaks <- seq(ZLIM[1], ZLIM[2], length.out=(length(COLORS)+1))
      poly <- vector(mode="list", length(COLORS))
      for(i in seq(poly)){
        poly[[i]] <- c(breaks[i], breaks[i+1], breaks[i+1], breaks[i])
      }
      par(mai=c(0.05,0.5,0.2,0.01))
      plot(1,1,t="n",col=NA,ylim=range(breaks), xlim=c(0,1), xaxt="n",yaxt="n",ylab="",bty="n",xaxs="i",yaxs="i") 
      #pBREAKS = BREAKS[seq(1,length(BREAKS),5)]+diff(BREAKS)[1]/2
      pBREAKS = seq(ceiling(min(BREAKS)/5)*5,floor(max(BREAKS)/5)*5,5)
      axis(2,at=pBREAKS,labels=pBREAKS,las=2,line=-0.3, tck=-.5,cex.axis=CEX,font=1)
      for(i in seq(poly)){
        if( i==1 ){ EDGE = c(0.5,0,1,0.5) }
        else if( i==length(poly) ){ EDGE = c(0,0.5,0.5,1) }
        else{ EDGE = c(0,0,1,1) }
        polygon(EDGE, poly[[i]], col=COLORS[i], border="black")
      }
      VARLAB=VAR[ivar]
      if(ivar==1){ VARLAB="Temperature"; VARUNIT=expression(paste("(C)")) }
      if(ivar==2){ VARLAB="Wind speed"; VARUNIT=expression(paste("(m s"^"-1",")")) }
      mtext(VARLAB,side=3, line=2.2, at=-.3,cex=CEX*.9)
      mtext("difference",side=3, line=1.5, at=-.3,cex=CEX*.9)
      mtext(VARUNIT,side=3, line=0.6, at=-.3,cex=CEX*.9)
      mtext(paste("[",REF," - other]",sep=""),side=3, line=0, at=-.2,cex=CEX*.85)
      if(PRINT) dev.off()
    }#ivar
  }#REF
}

# plot
if(0){
  PRINT=1
  for( REF in c("C15","T15") ){
    
    if(REF=="C15"){ IVAR=(1:2) }else{IVAR=1}
    IVAR=(1:2)
    for( ivar in IVAR ){
      WIDTH=6
      HEIGHT=3
      filename = list(T_2M=list(C15="fig03",T15="fig04"),FF=list(C15="fig05",T15="sup01"))[[VAR[ivar]]][[REF]]
      #if(PRINT){ pdf(file=paste("T:/cosmoPaper/figures/",filename,".pdf",sep=""),width=WIDTH,height=HEIGHT) }
      if(PRINT){ png(filename=paste("T:/cosmoPaper/figures/",filename,".png",sep=""), width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
      #if(PRINT){ png(filename=paste("T:/cosmoPaper/2dmm/paper/pics/",REF,"minusModell_",VAR[ivar],".png",sep=""), width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
      layout(cbind(matrix(rep(1:8,each=3),byrow=TRUE,ncol=4*3),rep(9,2),rep(9,2)))
      CEX=1
      par(mai=rep(0.01,4),omi=c(.05,.2,0.45,.02),cex=CEX)
      UNIT = c("K","m s^-1","hPa","K") #,"W m^-2","W m^-2","W m^-2","W m^-2")
      if(ivar%in%c(1,4)){
        BREAKS = seq(-11.5,11.5,1)
        ZLIM = range(BREAKS)
        COLORS = c(
          colorRampPalette(c("purple","blue","white","red","yellow"),space ="Lab")(length(BREAKS)-1)
        )
      } 
      if(ivar%in%c(2)){
        BREAKS = seq(-6.5,6.5,1)
        ZLIM = range(BREAKS)
        COLORS = c(
          colorRampPalette(c("blue","white","red"),space ="Lab")(length(BREAKS)-1)
        )
      } 
      firstYear=2014
      lastYear=2016
      for( yy in 1:2 ){
        for( xx in 1:4 ){
          if(yy==1){ month = c(10:12,1:3) }
          if(yy==2){ month = 4:9 }
          VS = c("AMPS","ERA5","ERA","ERA")[xx]
          XLAB = c("AMPS","ERA5","ERA-Interim","ERA-Interim")[xx]
          if(xx==4){
            REFtime = model[[ REF ]] [["time"]] %in% model[[ VS ]] [["time"]]
            VStime =  model[[ VS ]] [["time"]] %in% model[[ REF ]] [["time"]] [REFtime]
          }else{
            REFtime = model[[ REF ]] [["time"]] %in% model[[ VS ]] [["time"]] & model[[ REF ]] [["time"]] >= as.POSIXct(paste(firstYear,"/01/01",sep="")) & model[[ REF ]] [["time"]] <= as.POSIXct(paste(lastYear,"/12/31",sep=""))
            VStime =  model[[ VS ]] [["time"]] %in% model[[ REF ]] [["time"]] [REFtime]
          }
          ZZ = model[[ REF ]] [[VAR[ivar]]] [,, REFtime] - model[[ VS ]] [[VAR[ivar]]] [ ,, VStime ]

          mindex = as.numeric(strftime(model[[ VS ]] [["time"]]  [VStime], "%m")) %in% month
          z = apply( ZZ[,,mindex], 1:2, mean )
          #z = ZZ[,,1]
          if(ivar%in%c(3)) z = z/100
          # A = matrix(rep(1:16,each=2),ncol=4)
          # A = A[,rep(1:4,each=2)]
          reduce = function(data,factor){
            makeMean = function(x){
              dim(x)=c(factor,length(x)/factor)
              return(apply(x,2,mean))
            }
            return(apply(t(apply(data,1,makeMean)),2,makeMean))
          }
          # reduce(A,2)
          #if( any( z<ZLIM[1] | z>ZLIM[2] ) ){print("z out of ZLIM-range")}
          
          print(range(z,na.rm = TRUE))
          z[z<ZLIM[1]] = ZLIM[1]
          z[z>ZLIM[2]] = ZLIM[2]
          image(reduce(z,2), zlim=ZLIM,xaxt="n",yaxt="n",col=COLORS,breaks=BREAKS,asp=1)
          zz = is.na(z); zz[zz==FALSE] = NA
          if(VS=="AMPS") image(zz,xaxt="n",yaxt="n",col="gray",add=TRUE)
          #image(z*0, zlim=ZLIM,xaxt="n",yaxt="n",col=COLORS,breaks=BREAKS,asp=1)
          contour(reduce(FR_LAND,2),labels="",add=TRUE)
          box(lwd=1)
          
          if(yy==1){ mtext( XLAB ,side=3,line=0.1,font=1) }
          if(yy==1 && xx%in%1:3){ mtext( paste(firstYear,"-",lastYear,sep="") ,side=3,line=1,font=1) }
          if(yy==1 && xx==4){ mtext( "2002-2016" ,side=3,line=1,font=1) }
          if(xx==1){ 
            if(yy==1){ YLAB = "Summer" }
            if(yy==2){ YLAB = "Winter" }
            mtext( YLAB ,side=2,line=0.1,font=1) 
          }
          
        }#VS
      }#month
      breaks <- seq(ZLIM[1], ZLIM[2], length.out=(length(COLORS)+1))
      poly <- vector(mode="list", length(COLORS))
      for(i in seq(poly)){
        poly[[i]] <- c(breaks[i], breaks[i+1], breaks[i+1], breaks[i])
      }
      par(mai=c(0.05,0.5,0.15,0.01))
      plot(1,1,t="n",col=NA,ylim=range(breaks), xlim=c(0,1), xaxt="n",yaxt="n",ylab="",bty="n",xaxs="i",yaxs="i") 
      #pBREAKS = BREAKS[seq(1,length(BREAKS),5)]+diff(BREAKS)[1]/2
      pBREAKS = seq(ceiling(min(BREAKS)/5)*5,floor(max(BREAKS)/5)*5,5)
      axis(2,at=pBREAKS,labels=pBREAKS,las=2,line=-0.3, tck=-.5,cex.axis=CEX,font=1)
      for(i in seq(poly)){
        if( i==1 ){ EDGE = c(0.5,0,1,0.5) }
        else if( i==length(poly) ){ EDGE = c(0,0.5,0.5,1) }
        else{ EDGE = c(0,0,1,1) }
        polygon(EDGE, poly[[i]], col=COLORS[i], border="black")
      }
      VARLAB=VAR[ivar]
      if(ivar==1){ VARLAB="Temperature"; VARUNIT=expression(paste("bias (C)")) }
      if(ivar==2){ VARLAB="Wind speed"; VARUNIT=expression(paste("bias (m s"^"-1",")")) }
      mtext(REF,side=3, line=1.9, at=-.3,cex=CEX*.9)
      mtext(VARLAB,side=3, line=1.0, at=-.3,cex=CEX*.9)
      mtext(VARUNIT,side=3, line=-0.1, at=-.3,cex=CEX*.9)
      if(PRINT) dev.off()
    }#ivar
  }#REF
}


