library(Imap)
library(fields)
library(ncdf4)
library(data.table)
library(doParallel)
source("T:/cosmoPaper/map/geo2rot.R")
source("T:/cosmoPaper/map/rotateWind.R")
getTimeIndex = function(time,TLIM){
  return( which(TLIM[1]<=time&time<=TLIM[2]) )
}
limz = function(data,ZLIM){
  data[data>ZLIM[2]] = ZLIM[2]
  data[data<ZLIM[1]] = ZLIM[1]
  return(data)
}
windVec = function(dd,ff=1){
  alpha = ((dd+180)%%360)/180*pi
  u = sin(alpha) * ff
  v = cos(alpha) * ff
  return(V=list(u=u,v=v))
}
windDir = function(u, v){
  return( (atan2(u,v)*180/pi + 180)%%360 )
}

# SAVE ORG DATA approx 60-90 min (10-15 per RES)
if(0){
  
  TLIM = as.POSIXct(c("2015/12/24 00:00","2016/01/30 00:00"),tz="UTC")
  spinup = FALSE
  load("T:/cosmoPaper/lidar/lidardata/shipPS96.Rdata")
  ship$time[] = round(ship$time[],"hour")
  SHIP = data.table(
    time = ship[,mean(lon),by=time]$time,
    lon = ship[,mean(lon),by=time]$V1,
    lat = ship[,mean(lat),by=time]$V1
  )
  meta = SHIP[getTimeIndex(SHIP$time,TLIM)]
  
  nproc=8
  if(nproc!=0){ cl <- makeCluster(nproc, outfile=""); registerDoParallel(cl) }
  RES = c("lidar","C01","T01","C05","T05","C15","T15","ERA5","ERAInterim")
  RES = c("ERAInterim")
  foreach( ipar = seq(RES), .packages=c("data.table","ncdf4") ) %dopar%{
    RES = RES[ipar]
    
    ### LIDAR ###
    if( RES=="lidar" ){
      #print("loading Lidar")
      lidar = read.table("T:/cosmoPaper/lidar/lidardata/PS96_lidar.tab",stringsAsFactors = FALSE,sep="~",nrow=100)
      lidar = read.table("T:/cosmoPaper/lidar/lidardata/PS96_lidar.tab",sep="\t",header=TRUE,skip=which(lidar$V1=="*/"),stringsAsFactors = FALSE)
      meta = data.table(time = as.POSIXct(lidar$Date.Time,"%Y-%m-%dT%H:%M",tz="UTC"))
      lidar = lidar[TLIM[1]<=meta$time&meta$time<=TLIM[2],]
      meta = meta[TLIM[1]<=meta$time&meta$time<=TLIM[2]]
      meta$lon = lidar$Longitude
      meta$lat = lidar$Latitude
      
      #ff = as.matrix(lidar[,grep("ff",names(lidar))])
      #dd = as.matrix(lidar[,grep("dd",names(lidar))])
      #fd = as.matrix(lidar[,grep("FD",names(lidar))])
      #bb = as.matrix(lidar[,grep("bb",names(lidar))])
      
      model = list(
        name = "lidar",
        meta = meta,
        HH = matrix(seq(50,1000,50),nrow=length(meta$time),ncol=20,byrow=TRUE),
        ff = as.matrix(lidar[,grep("ff",names(lidar))]),
        dd = as.matrix(lidar[,grep("dd",names(lidar))])
      )
    }
    if( RES=="old_lidar" ){
      #print("loading Lidar")
      lidar = read.table("T:/cosmoPaper/lidar/lidardata/PS96.txt",sep="\t",header=TRUE,stringsAsFactors = FALSE)
      meta = data.table(time = as.POSIXct(lidar$time,tz="UTC"))
      lidar = lidar[TLIM[1]<=meta$time&meta$time<=TLIM[2],]
      meta = meta[TLIM[1]<=meta$time&meta$time<=TLIM[2]]
      meta$lon = lidar$lon
      meta$lat = lidar$lat
      
      # #over iceberg
      # meta$lon = -41
      # meta$lat = -75.9
      # #over polynya
      # meta$lon = -42.5
      # meta$lat = -75.55
      # print("over iceberg")
      
      # out = geo2rot(0,12.5,meta$lon,meta$lat,180)
      # meta$rlon = (out$x)
      # meta$rlat = (out$y)
      model = list(
        name = "lidar",
        meta = meta,
        HH = matrix(seq(50,2000,50),nrow=length(meta$time),ncol=40,byrow=TRUE),
        ff = as.matrix(lidar[,grep("ff",names(lidar))]),
        dd = as.matrix(lidar[,grep("dd",names(lidar))])
      )
    }
    
    ### COSMO ###
    if( RES %in% c("C01","T01","C05","T05","C15","T15") ){
      #print(RES)
      if(RES=="T15"|RES=="T01"|RES=="C01"|RES=="T05") LETTER = "R" else LETTER = "S"
      FF = DD = HH = matrix(NA,nrow=length(meta$time),ncol=60)
      Meta = meta
      for( i in seq(meta$lon) ){
        # open constantfile
        if( spinup & as.numeric(strftime(meta$time[i],"%H",tz="UTC"))>=18 ){
          filename = paste(LETTER,":/COSMO_",RES,"/",strftime(meta$time[i]+3600*24,"%Y%m%d",tz="UTC"),"/out01/lffd",strftime(meta$time[i],"%Y%m%d",tz="UTC"),"18c.nc",sep="")
        }else{
          filename = paste(LETTER,":/COSMO_",RES,"/",strftime(meta$time[i],"%Y%m%d",tz="UTC"),"/out01/lffd",strftime(meta$time[i]-3600*24,"%Y%m%d",tz="UTC"),"18c.nc",sep="")
        }

        nc <- nc_open(filename)
        lon <- ncvar_get(nc,"lon")
        lat <- ncvar_get(nc,"lat")
        hhl = ncvar_get(nc,"HHL")
        nc_close(nc)
        # find cosmo grid points nearest to station
        dist = sqrt(((lon-meta$lon[i])*cos(meta$lat[i]/180*pi))^2+(lat-meta$lat[i])^2)
        coords = which(dist == min(dist) , arr.ind = TRUE)[1,]
        
        # open 3D var file
        if(spinup & as.numeric(strftime(meta$time[i],"%H",tz="UTC"))>=18 ){
          filename = paste(LETTER,":/COSMO_",RES,"_out01/",strftime(meta$time[i]+3600*24,"%Y%m%d",tz="UTC"),"/out01/lffd",strftime(meta$time[i],"%Y%m%d%H",tz="UTC"),".nc",sep="")
        }else{
          filename = paste(LETTER,":/COSMO_",RES,"_out01/",strftime(meta$time[i],"%Y%m%d",tz="UTC"),"/out01/lffd",strftime(meta$time[i],"%Y%m%d%H",tz="UTC"),".nc",sep="")
        }
        
        if(file.exists(filename)){
          nc = nc_open(filename)
          U = apply(ncvar_get(nc,"U",start=c(coords,1,1),count=c(2,1,60,1)),2,mean)
          V = apply(ncvar_get(nc,"V",start=c(coords,1,1),count=c(1,2,60,1)),2,mean)
          nc_close(nc)
          # pollon = 0;  pollat = 12.5;  polgam = 180
          # point = rot2geo(pollon, pollat, rlon[cx], rlat[cy], polgam)
          geouv = uvrot2geo(U,V,lon[coords[1],coords[2]],lat[coords[1],coords[2]])
          FF[i,] = sqrt(geouv$u^2+geouv$v^2)
          DD[i,] = (atan2(geouv$u,geouv$v)*180/pi + 180)%%360
        }else{
          FF[i,] = NA
          DD[i,] = NA
        }
        HH[i,] = (hhl[coords[1],coords[2],1:60]+hhl[coords[1],coords[2],2:61])/2
        Meta$lon[i] = lon[coords[1],coords[2]]
        Meta$lat[i] = lat[coords[1],coords[2]]

      }
      model = list(
        name = RES,
        meta = Meta,
        HH = HH[,60:1],
        ff = FF[,60:1],
        dd = DD[,60:1]
      )
    }
    
    ### ERA ###
    if( RES %in% c("ERA5","ERAInterim") ){
      #print(RES)
      print("Height should be correct up to 2000m (error of 1m around that height)")
      if(RES=="ERAInterim"){
        filename = paste("T:/",RES,"/caf",strftime(meta$time[1],"%Y%m%d",tz="UTC"),"00.nc",sep="")
      }else if(RES=="ERA5"){
        filename = paste("T:/",RES,"/caf",strftime(meta$time[1],"%Y%m%d",tz="UTC"),"00.nc",sep="")
      }
      nc <- nc_open(filename)
      lon <- ncvar_get(nc,"lon")
      lat <- ncvar_get(nc,"lat")
      akm = ncvar_get(nc,"akm")
      bkm = ncvar_get(nc,"bkm")
      nc_close(nc)
      N = length(akm)
      FF = DD = HH = matrix(NA,nrow=length(meta$time),ncol=N)
      Meta = meta
      for( i in seq(meta$lon) ){
        # find era grid points nearest to station
        coords = apply( abs(rbind(lon-meta$lon[i],lat-meta$lat[i])), 1, which.min )
        if(RES=="ERAInterim"){
          Hour = formatC((round((as.numeric(strftime(meta$time[i],"%H",tz="UTC"))+.5)/6)*6)%%24,flag="0",width=2)
          if(Hour=="00"){ 
            Date = strftime(round(meta$time[i],"day"),"%Y%m%d",tz="UTC")
          }else{
            Date = strftime(meta$time[i],"%Y%m%d",tz="UTC")
          }
          filename = paste("T:/ERAInterim/caf",Date,Hour,".nc",sep="")
        }else if(RES=="ERA5"){
          filename = paste("T:/ERA5/caf",strftime(meta$time[i],"%Y%m%d%H",tz="UTC"),".nc",sep="")
        }
        nc = nc_open(filename)
        U = ncvar_get(nc,"U",start=c(coords,1,1),count=c(1,1,N,1))
        V = ncvar_get(nc,"V",start=c(coords,1,1),count=c(1,1,N,1))
        PS = ncvar_get(nc,"PS",start=c(coords,1),count=c(1,1,1))
        FIS = ncvar_get(nc,"FIS",start=c(coords,1),count=c(1,1,1)) 
        nc_close(nc)
        PS = 101325 * (1-(0.0065* FIS/9.80665 /288.15))^5.255
        HH[i,] = 288.15/0.0065 * (1-( (akm + bkm * c(PS)) /101325)^(1/5.255))
        FF[i,] = sqrt(U^2+V^2)
        DD[i,] = (atan2(U,V)*180/pi + 180)%%360
        Meta$lon[i] = lon[coords[1]]
        Meta$lat[i] = lat[coords[2]]
      }
      model = list(
        name = RES,
        meta = Meta,
        HH = HH[,N:1],
        ff = FF[,N:1],
        dd = DD[,N:1]
      )
      
    }
    
    ### save ###
    save(model,file=paste("T:/cosmoPaper/lidar/rdata/",RES,".Rdata",sep=""))
    
  }
  if(nproc!=0){ stopCluster(cl) }

}
# interpolate ORG DATA
if(0){
  
  MODEL = list();
  RES = c("lidar","C01","T01","C05","T05","C15","T15","ERA5","ERAInterim")
  for( i in seq(RES) ){
    load(paste("T:/cosmoPaper/lidar/rdata/",RES[i],".Rdata",sep=""))
    MODEL[[i]] = model
  }
  model=MODEL

  newH = seq(50,1000,50)
  FF = DD = matrix(NA,nrow=length(model[[2]]$meta$time),ncol=length(newH))

  for( im in seq(model)[-1] ){
    if(model[[im]]$name=="ERAInterim"){ model[[im]]$name="ERA-Interim" }
    print(model[[im]]$name)
    time = model[[im]]$meta$time
    
    for( it in 1:length(time) ){
      
      H = model[[im]]$HH[it,]
      uv = windVec(model[[im]]$dd[it,],model[[im]]$ff[it,])
      
      if(all(is.na(uv$u))){FF[it,]=DD[it,]=NA; next()}
      u = approx(H,uv$u,newH)$y
      v = approx(H,uv$v,newH)$y
      
      FF[it,] = sqrt(u^2+v^2)
      DD[it,] = windDir(u,v)
    }
    model[[im]]$ff = FF
    model[[im]]$dd = DD
    model[[im]]$HH = matrix(newH,nrow=length(time),ncol=20,byrow=TRUE)
  }
  
}

# case study A
###TLIM = as.POSIXct(c("2016/01/16 12:00","2016/01/17 18:00"),tz="UTC")
#TLIM = as.POSIXct(c("2016/01/16 18:00","2016/01/17 23:00"),tz="UTC")
# case study B
#TLIM = as.POSIXct(c("2016/01/20 00:00","2016/01/31 00:00"),tz="UTC")
# case study C
#TLIM = as.POSIXct(c("2015/12/30 12:00","2016/01/02 12:00"),tz="UTC")


# Map
if(0){
  print("loading Map")
  filename = paste("S:/COSMO_C05/",strftime(mean(TLIM),"%Y%m%d",tz="UTC"),"/out02/lffd",strftime(mean(TLIM),"%Y%m%d",tz="UTC"),"00.nc",sep="")
  nc <- nc_open(filename)
  rlon <- ncvar_get(nc,"rlon")
  rlat <- ncvar_get(nc,"rlat")
  hice = ncvar_get(nc,"H_ICE")
  nc_close(nc)
  hice[is.na(hice)] = -1
  out = geo2rot(0,12.5,meta$lon,meta$lat,180)
  boundary = 3
  subx = (which.min(abs(rlon-min(out$x)))-boundary):(which.min(abs(rlon-max(out$x)))+boundary)
  suby = (which.min(abs(rlat-min(out$y)))-boundary):(which.min(abs(rlat-max(out$y)))+boundary)
  par(mfrow=c(1,1),omi=c(0.2,.4,.2,.4),mai=rep(0.,4),cex=1.5)
  image(rlon[subx],rlat[suby],hice[subx,suby],asp=1,col=colorRampPalette(c("gray50","dodgerblue","gray95"))(4),xaxt="n",yaxt="n",xlab="",ylab="",zlim=c(-1,1))
  text_legend = NULL
  LWD = 3
  for( m in seq(model) ){
    out = geo2rot(0,12.5,model[[m]]$meta$lon,model[[m]]$meta$lat,180)
    lines(out$x,out$y,type="b",col=m,lwd=LWD)
    text_legend = c(text_legend,model[[m]]$name)
  }
  legend("topright",text_legend,col=1:length(model),lwd=LWD,lty=1,pch=1)
}

# plot statistic / make table
if(1){
  PRINT=1
  
  TABLE = data.table(name=c("ff_bias","ff_RMSE","dd_bias","dd_RMSE"))
  STAT = matrix(NA,nrow=4,ncol=8)
  
  WIDTH=6
  HEIGHT=4
  if(PRINT){ pdf(file=paste("T:/cosmoPaper/figures/sup07.pdf",sep=""), width=WIDTH ,height=HEIGHT) }
  #if(PRINT){ png("T:/cosmoPaper/figures/sup07.png",width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
  #if(PRINT){ png("T:/cosmoPaper/lidar/statistic.png",width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
  par(mfrow=c(1,2),omi=rep(.1,4),mai=c(.5,.6,.1,.2))
  H = seq(50,1000,50)
  COL = c("dodgerblue4","chartreuse4","dodgerblue3","chartreuse3","dodgerblue1","chartreuse1","firebrick4","firebrick1")
  LWD = rep(3,length(COL))
  LTY = c(2,1,2,1,2,1,3,3)
  for( ffdd in c("ff","dd") ){
    YLIM = c(0,1150)
    YAT = YmAT = seq(YLIM[1],YLIM[2],500)
    NAMES = NULL
    if(ffdd=="ff"){ XLIM = c(-1,4)  ;XAT = XmAT = seq(-1,4,1) ;MAIN=expression(paste("Wind speed (m s"^"-1",")")) }
    if(ffdd=="dd"){ XLIM = c(-20,50);XAT = seq(-20,50,20); XmAT = seq(-20,50,10)  ;MAIN=expression(paste("Wind direction ()")) }
    plot(0,0,col=0,xlim=XLIM,ylim=YLIM,xlab="",ylab="",xaxt="n",yaxt="n")
    ## axis 1 ##
    axis(1,at=XmAT,tck=-0.02,labels=FALSE)
    axis(1,at=XAT,tck=-0.04,labels=FALSE)
    axis(1,at=XAT,line=-.6,lty=0,las=1)
    mtext(text=MAIN,side=1,line=1.8,cex=1,font=2)
    ## axis 2 ##
    axis(2,at=YmAT,tck=-0.02,labels=FALSE)
    axis(2,at=YAT,tck=-0.04,labels=FALSE)
    axis(2,at=YAT,line=-.5,lty=0,las=2)
    mtext(text="Height (m)",side=2,line=2.5,cex=1,font=2)
    ## axis 3 ##
    axis(3,at=XmAT,tck=-0.02,labels=FALSE)
    axis(3,at=XAT,tck=-0.04,labels=FALSE)
    ## axis 4 ##
    axis(4,at=YmAT,tck=-0.02,labels=FALSE)
    axis(4,at=YAT,tck=-0.04,labels=FALSE)
    abline(v=0,col="gray",lwd=5)
    for( m in 2:length(model) ){
        NAMES = c(NAMES,model[[m]]$name)
        diff = model[[m]][[ffdd]][model[[m]]$meta$time%in%model[[1]]$meta$time]-model[[1]][[ffdd]]
        diff = diff + 360*(diff< -180) -360*(diff>180)
        N = apply(diff*0+1,2,sum,na.rm=TRUE)
        BIAS = apply(diff,2,mean,na.rm=TRUE)
        RMSE = sqrt(apply(diff^2,2,mean,na.rm=TRUE))
        if(any(N<30)){print(m)}
        BIAS[N<30] = NA
        RMSE[N<30] = NA
        lines(BIAS,H,col=COL[m-1],lwd=LWD[m-1],lty=LTY[m-1])
        lines(RMSE,H,col=COL[m-1],lwd=LWD[m-1],lty=LTY[m-1])
        points(mean(diff,na.rm=TRUE),0,col=COL[m-1],lwd=LWD[m-1],pch=2)
        points(sqrt(mean(diff^2,na.rm=TRUE)),0,col=COL[m-1],lwd=LWD[m-1],pch=6)
        #print(c(model[[m]]$name, round(mean(diff,na.rm=TRUE),1), round(sqrt(mean(diff^2,na.rm=TRUE)),1) ))
        STAT[list(ff=1,dd=3)[[ffdd]]+0,m-1] = round(mean(diff,na.rm=TRUE),list(ff=1,dd=0)[[ffdd]])
        STAT[list(ff=1,dd=3)[[ffdd]]+1,m-1] = round(sqrt(mean(diff^2,na.rm=TRUE)),list(ff=1,dd=0)[[ffdd]])
    }
    if(ffdd=="ff"){
      text(c(0,2),rep(1100,2),c("Bias","RMSE"))
    }
  }
  NAMES[8]="ERA-I"
  legend(XLIM[1],1200,legend=NAMES,col=COL,lwd=LWD,lty=LTY,cex=.6,bty="n",ncol = 3)
  if(PRINT) dev.off()
  
  for( m in 2:length(model) ){
    TABLE[[model[[m]]$name]] = STAT[,m-1]
  }
  
  write.table(TABLE,file=paste("T:/cosmoPaper/lidar/paper/table.txt",sep=""),row.names = FALSE,quote=FALSE,sep=" & ",eol=" \\\\\n")
  #write.table(TABLE,file=paste("T:/cosmoPaper/lidar/paper/table.txt",sep=""),row.names = FALSE,quote=FALSE,sep="\t",eol="\n")
  
  
}

# plot raso test
if(0){
  PRINT=0
  
  COL = c("dodgerblue4","chartreuse4","dodgerblue3","chartreuse3","dodgerblue1","chartreuse1","firebrick4","firebrick1")
  LWD = rep(3,length(COL))
  LTY = c(2,1,2,1,2,1,3,3)
  
  INTERP=50
  vCOL = c("purple","orange")
  vPCH = 19
  dCOL = "gray60"
  CEX=1.3
  for( xPS in c(3) ){
    
    WIDTH=7
    HEIGHT=4
    #if(PRINT){ pdf(paste("T:/cosmoPaper/lidar/paper/caseStudyRaso",xPS,".pdf",sep=""), width=WIDTH ,height=HEIGHT) }
    if(PRINT){ png(paste("T:/cosmoPaper/lidar/paper/caseStudyRaso",xPS,".png",sep=""),width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
    par(mfrow=c(1,3),omi=c(.8,.8,.9,.2),mai=rep(0,4),cex=CEX)
    for( ffdd in c("T","FF","DD") ){
      
      if(xPS==3){
        YLIM=c(0,1000)
        PS="PS96"; SNR=-20; TLIM = as.POSIXct(c("2016/01/17 07:00"),tz="UTC")
        if(ffdd=="FF"){ XLIM = c(5,14) }
        if(ffdd=="DD"){ XLIM = c(60,120) }
        if(ffdd=="T"){ XLIM = c(-2,7); LABELS = seq(0,6,1) }
      }
      # load VAD Files
      rDataDirectory = "T:/cosmoPaper/lidar/lidardata"
      getVAD = function(PS,INTERP,RASO,RP,SNR,LO,SO,AV,LY,TO,rpSD,ySD,VER){
        if(RASO){
          prefix = "/VADRASO/VADRASO"
        }else{
          prefix = "/VAD/VAD"
        }
        load(paste(rDataDirectory,prefix,"_INTERP",INTERP,"_RP",RP,"_SNR",SNR,"_LO",LO,"_SO",SO,"_AV",AV,"_LY",LY,"_TO",TO,"_rpSD",rpSD,"_ySD",ySD,"_VER",VER,"_",PS,".Rdata",sep=""))
        return(VAD)
      }
      VAD = getVAD(PS,INTERP,1,1,SNR,1.2,1,1,0,1,0.5,2,1)
      getRasoData = function(PS){
        load(file=paste(rDataDirectory,"/raso",PS,".Rdata",sep=""))
        return(raso)
      }
      raso = getRasoData(PS)
      iraso = which( TLIM-3600 < raso$meta$time & raso$meta$time < TLIM+3600 )
      ivad = last(which((raso$meta$time[iraso]-VAD$meta$time)>0))
      
      plot(0,0,col=0,xlim=XLIM,ylim=YLIM,yaxs="i",xaxs="i",bty="n",xaxt="n",yaxt="n")
      if(ffdd=="T"){
        axis(2)
        axis(1)
        mtext("Height (m)",side=2,line = 2,cex=CEX)
        mtext("Pot. temperature (C)",side=1,line = 2,cex=CEX)
        lines(raso$data[[iraso]][["theta"]],raso$data[[iraso]][["g"]],col="black",lwd=5)
        y = raso$data[[iraso]][["t"]]-raso$data[[iraso]][["td"]]
        AT = seq(c(XLIM%*%c(.9,.1)),c(XLIM%*%c(.4,.6)),length.out=length(LABELS))
        y = y / last(LABELS) * diff(range(AT)) + first(AT)
        lines(y,raso$data[[iraso]][["g"]],lwd=5,col=dCOL)
        dewticks=c(1,3,5,7)
        axis(3,at=AT[dewticks],labels=LABELS[dewticks] ,col=dCOL,col.axis = dCOL)
        mtext(expression("Dew-point spread (C)"),side=3,line = 2,col=dCOL,cex=CEX)
        
        # get temperature
        for( im in 2:(length(model)-2) ){
          # constant
          RES = model[[im]]$name
          if(RES=="T15"|RES=="T01"|RES=="C01"|RES=="T05") LETTER = "R" else LETTER = "S"
          filename = paste(LETTER,":/COSMO_",RES,"/",strftime(round(raso$meta$time[iraso],"hour"),"%Y%m%d",tz="UTC"),"/out01/lffd",strftime(round(raso$meta$time[iraso],"hour")-3600*24,"%Y%m%d",tz="UTC"),"18c.nc",sep="")
          nc <- nc_open(filename)
          lon <- ncvar_get(nc,"lon")
          lat <- ncvar_get(nc,"lat")
          hhl = ncvar_get(nc,"HHL")
          h = (hhl[,,1:60]+hhl[,,2:61])/2
          vcoord = ncvar_get(nc,"vcoord")
          p0sl = ncatt_get(nc,"vcoord","p0sl")$value
          t0sl = ncatt_get(nc,"vcoord","t0sl")$value
          dt0lp = ncatt_get(nc,"vcoord","dt0lp")$value
          delta_t = ncatt_get(nc,"vcoord","delta_t")$value
          h_scal = ncatt_get(nc,"vcoord","h_scal")$value
          g = 9.80665#9.807
          R = 287.05
          t00 = t0sl - delta_t
          # pref = p0sl * exp( -g*h_scal/ R/t00 * log( (exp(vcoord/h_scal)*t00 + delta_t)/(t00 + delta_t)) )
          pref = p0sl * exp( -g*h_scal/ R/t00 * log( (exp(h/h_scal)*t00 + delta_t)/(t00 + delta_t)) )
          nc_close(nc)
          
          # find cosmo grid points nearest to station
          dist = sqrt(((lon-raso$meta$lon[iraso])*cos(raso$meta$lat[iraso]/180*pi))^2+(lat-raso$meta$lat[iraso])^2)
          coords = which(dist == min(dist) , arr.ind = TRUE)[1,]
          filename = paste(LETTER,":/COSMO_",RES,"_out01/",strftime(round(raso$meta$time[iraso],"hour"),"%Y%m%d",tz="UTC"),"/out01/lffd",strftime(round(raso$meta$time[iraso],"hour"),"%Y%m%d%H",tz="UTC"),".nc",sep="")
          nc = nc_open(filename)
          T = ncvar_get(nc,"T",start=c(coords,1,1),count=c(1,1,60,1))
          PP = ncvar_get(nc,"PP",start=c(coords,1,1),count=c(1,1,60,1))
          p = (PP + pref[coords[1],coords[2],])/100
          nc_close(nc)
          index = last(which((raso$meta$time[iraso]-model[[im]]$meta$time)>0))+1
          print(model[[im]]$meta$time[index])
          lines(model[[im]]$ff[index,],model[[im]]$HH[index,],col=COL[im-1],lty=LTY[im-1],lwd=LWD[im-1])
        }

      }

      if(ffdd=="FF"){
        axis(3)
        mtext(expression(paste("        Wind speed (m s"^"-1",")")),side=3,line = 2,cex=CEX)
        lines(raso$data[[iraso]][[tolower(ffdd)]],raso$data[[iraso]][["g"]],col="black",lwd=5)
        for( im in 2:length(model) ){
          index = last(which((raso$meta$time[iraso]-model[[im]]$meta$time)>0))+1
          print(model[[im]]$meta$time[index])
          lines(model[[im]]$ff[index,],model[[im]]$HH[index,],col=COL[im-1],lty=LTY[im-1],lwd=LWD[im-1])
        }
        points(t(VAD[[ffdd]][ivad+(0:1),]),rep(VAD$H,2),col=rep(vCOL,each=length(VAD$H)),pch=vPCH,cex=.7)
        mtext(strftime(VAD$meta[ivad+(0)]$time,"Lidar %H:%M UTC",tz="UTC"),col=vCOL[1],side=1,line=0,cex=CEX,font=2)
        mtext(strftime(raso$meta[iraso]$time,"  RS  %H:%M UTC",tz="UTC"),side=1,line=1,cex=CEX,font=2)
        mtext(strftime(VAD$meta[ivad+(1)]$time,"Lidar %H:%M UTC",tz="UTC"),col=vCOL[2],side=1,line=2,cex=CEX,font=2)
      }
      if(ffdd=="DD"){
        NAMES=NULL
        axis(1,at=seq(XLIM[1],XLIM[2],20))
        mtext("Wind direction ()",side=1,line = 2,cex=CEX)
        y = raso$data[[iraso]][[tolower(ffdd)]]
        y = y - (y>XLIM[2])*360
        lines(y,raso$data[[iraso]][["g"]],col="black",lwd=5)
        for( im in 2:length(model) ){
          NAMES = c(NAMES,model[[im]]$name)
          index = last(which((raso$meta$time[iraso]-model[[im]]$meta$time)>0))
          lines(model[[im]]$dd[index,],model[[im]]$HH[index,],col=COL[im-1],lty=LTY[im-1],lwd=LWD[im-1])
        }
        y = t(VAD[[ffdd]][ivad+(0:1),])
        y = y - (y>XLIM[2])*360
        points(y,rep(VAD$H,2),col=rep(vCOL,each=length(VAD$H)),pch=vPCH,cex=.7)
        #mtext(strftime(raso$meta[iraso]$time,"    %Y/%m/%d",tz="UTC"),side=3,line=0.7,cex=2,font=2)
        NAMES[8]="ERA-I"
        legend(XLIM[1],YLIM[2],legend=NAMES,col=COL,lwd=LWD,lty=LTY,cex=.45,ncol = 3)
      }

    }#ffdd
    if(PRINT) dev.off()
  }#xPS
  
}

# plot diff
if(0){
  PRINT=1
  if(PRINT) png("T:/cosmoPaper/lidar/test.png",height=900,width=900,res=100)
  par(mfrow=c(length(model)+1+1,2),omi=rep(.01,4),mai=c(0,0,0,0))
  #par(mfrow=c(length(model)*2+2,1),omi=c(0,.4,.2,.4),mai=rep(0.,4))
  for( ffdd in c("ff","dd") ){
    for( m in c(1,-1,2:length(model)) ){
      if(m==2){plot(0,0,col=0,xlim=TLIM,ylim=c(0,1000),xaxt="n",yaxt="n"); text(mean(TLIM),800,"C01 crashed",font=2,cex=2)}
      par(cex=1)
      #for( ffdd in c("ff","dd") ){
      defaultSMALLPLOT=c(.2,.95,.45,.60)
      if(ffdd=="ff"){ H=1/2; N=11; M=-5; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c("purple",colorRampPalette(c("blue","white","red"))(N)[-c(1,N)],"magenta"); TICKS=M+seq(0,N*H*2,H*4) }
      if(ffdd=="dd"){ H=50/10; N=11; M=-50; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c("purple",colorRampPalette(c("blue","white","red"))(N)[-c(1,N)],"magenta"); TICKS=M+seq(0,N*H*2,H*4) }
      #  COL = c("purple",tim.colors(diff(ZLIM)/(10^(ffdd=="dd"))-1),"purple")
      if(m>0){
        image(range(model[[m]]$meta$time),1:2,(1:2)%*%t(1:2)*0-999,xaxt="n",yaxt="n",xlab="",ylab="",xlim=TLIM,ylim=c(0,1000),breaks=BREAKS,col=COL)
        for( i in seq(meta$lon) ){
          zz = limz(model[[m]][[ffdd]][c(i,NA),]-model[[1]][[ffdd]][c(i,NA),],ZLIM) 
          image(model[[m]]$meta$time[c(i,i)]+c(0,diff(as.numeric(model[[m]]$meta$time))[1]),model[[m]]$H[i,],zz,breaks=BREAKS,col=COL,add=TRUE)
        }
        box()
        text( TLIM%*%c(.8,.2), 750 , model[[m]]$name ,cex=1.5, font=2)
        axis(2,seq(0,1000,250),tck=.1,label=FALSE,lwd=2) 
        axis(4,seq(0,1000,250),tck=.1,label=FALSE,lwd=2) 
        axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="1 hour"),labels=F,lwd=1,tcl=0.15)
        axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="6 hour"),labels=F,lwd=2,tcl=0.25)
        axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="1 hour"),labels=F,lwd=1,tcl=0.15)
        axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="6 hour"),labels=F,lwd=2,tcl=0.25)
        if(m==3){
          axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="6 hours")[-1],format = "%H:%M",labels=TRUE,tcl=-0.25,padj=1,las=1,lwd=2,font=2)
          axis.POSIXct(side=3,at = seq(round(TLIM[1],"day"),round(TLIM[2],"day"),by="24 hours"),format = "%m/%d",labels=TRUE,tcl=-0.25,padj=.0,las=1,lwd=2,font=2)
          mtext("time",3,font=2,line=2)
        }
        if(m==1){
          axis(4,seq(250,750,250),tck=-.1,label=TRUE,las=2,hadj=.3,lwd=2,font=2) 
          mtext("height (m)",4,font=2,line=1.9)
        }
      }else{
        plot(0,0,col=0,xaxt="n",yaxt="n",xlab="",ylab="",bty="n")
        image.plot(2,smallplot=SMALLPLOT,horizontal=TRUE,legend.lab=list(ff="Wind speed (m/s)",dd="Wind direction ()")[[ffdd]],legend.cex=1.3,legend.line=-2.4, breaks=BREAKS,col=COL,legend.only = TRUE,add=TRUE,axis.args=list( at=TICKS,padj=-3.9, tck=1, labels=names(TICKS),font=2 ))
      }
    }
  }
  if(PRINT) dev.off()
}

# half_old plot case study B
if(0){
  PRINT=1
  
  WIDTH=8
  HEIGHT=8
  for( fs in 1:2 ){
    for( xCase in 3 ){
      filename = list(a="fig16",b="sup08")[[fs]]
      MODELS = list(a=c(1,7,8,9,-1),b=c(1,3,5,7,-1))[[fs]]
      #if(PRINT){ pdf(paste("T:/cosmoPaper/figures/",filename,".pdf",sep=""), width=WIDTH ,height=HEIGHT) }
      if(PRINT){ png(paste("T:/cosmoPaper/figures/",filename,".png",sep=""),width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
      #if(PRINT){ png(paste("T:/cosmoPaper/lidar/paper/caseStudy",xCase,".png",sep=""),width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
      par(mfrow=c(10,1),omi=c(.01,.5,.01,.01),mai=c(0,0,0,0))
      FONT = 2
      #par(mfrow=c(length(model)*2+2,1),omi=c(0,.4,.2,.4),mai=rep(0.,4))
      for( ffdd in c("ff","dd") ){
        for( m in MODELS ){
          if(m==2){plot(0,0,col=0,xlim=TLIM,ylim=c(0,1000),xaxt="n",yaxt="n"); text(mean(TLIM),800,"C01 crashed",font=2,cex=2)}
          par(cex=1)
          #for( ffdd in c("ff","dd") ){
          if(m==-1&ffdd=="dd"){
            plot(0,0,col=0,xlim=TLIM,ylim=c(0,1000),xaxt="n",yaxt="n",xlab="",ylab="",bty="n")
            #image.plot(2,smallplot=SMALLPLOT,horizontal=TRUE,legend.lab=list(ff="Wind speed (m/s)",dd="Wind direction ()")[["ff"]],legend.cex=1.3,legend.line=-2.4, breaks=BREAKS,col=COL,legend.only = TRUE,add=TRUE,axis.args=list( at=TICKS,padj=-3.9, tck=1, labels=names(TICKS),font=2 ))
            image.plot(2,smallplot=SMALLPLOT,horizontal=TRUE,legend.lab=list(ff="Wind speed (m/s)",dd="Wind direction ()")[["ff"]],legend.cex=1.3,legend.line=0.8, breaks=BREAKS,col=COL,legend.only = TRUE,add=TRUE,axis.args=list( at=TICKS,padj=-1.9, tck=1, labels=names(TICKS),font=2 ))
          }
          defaultSMALLPLOT=c(.2,.95,.45,.60)
          if(xCase==3){
            # case study B
            TLIM = as.POSIXct(c("2016/01/20 00:00","2016/01/30 00:00"),tz="UTC")
            if(ffdd=="ff"){H=1/100; N=12.5*100+1; M=0; SMALLPLOT=c(.1,.45,.7,.9); ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c(tim.colors(N)[-N],"purple"); TICKS=M+seq(0,N*H*2,H*5*100) }
            if(ffdd=="dd"){H=45/8/100; N=10*100; M=180; SMALLPLOT=c(.55,.9,.1,.30); ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c("purple",tim.colors(N)[c(-1,-N)],"purple"); TICKS=M+seq(0,N*H*2,H*8*100) }
            if(ffdd=="dd"){H=18/100; N=10*100+1; M=0; SMALLPLOT=c(.55,.9,.1,.30); ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=colorRampPalette(c("purple","firebrick1","yellow","deepskyblue","purple"))(N); TICKS=M+seq(0,N*H*2,H*5*100) }
          }
          
          #  COL = c("purple",tim.colors(diff(ZLIM)/(10^(ffdd=="dd"))-1),"purple")
          if(m>0){
            image(range(model[[m]]$meta$time),1:2,(1:2)%*%t(1:2)*0-999,xaxt="n",yaxt="n",xlab="",ylab="",xlim=TLIM,ylim=c(0,1000),breaks=BREAKS,col=COL)
            # for( i in seq(model[[m]]$meta$lon) ){
            #   image(model[[m]]$meta$time[c(i,i)]+c(0,diff(as.numeric(model[[m]]$meta$time))[1]),model[[m]]$H[i,],limz(model[[m]][[ffdd]][c(i,NA),],ZLIM),breaks=BREAKS,col=COL,add=TRUE)
            # }
            box()
            text( TLIM%*%c(.5,.5), 750 , model[[m]]$name ,cex=1.5, font=FONT)
            axis(2,seq(0,1000,250),tck=.1,label=FALSE,lwd=2) 
            axis(4,seq(0,1000,250),tck=.1,label=FALSE,lwd=2) 
            #axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="1 hour"),labels=F,lwd=1,tcl=0.15)
            axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="6 hour"),labels=F,lwd=2,tcl=0.25)
            #axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="1 hour"),labels=F,lwd=1,tcl=0.15)
            axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="6 hour"),labels=F,lwd=2,tcl=0.25)
            if(m==MODELS[4]&ffdd=="dd"){
              #axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="6 hours")[-1],format = "%H:%M",labels=TRUE,tcl=-0.25,padj=1,las=1,lwd=2,font=2)
              axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="day")[-c(1,length(seq(TLIM[1],TLIM[2],by="day")))],format = "%d",labels=TRUE,tcl=-0.25,padj=-1,las=1,lwd=2,font=FONT)
              #axis.POSIXct(side=3,at = seq(round(TLIM[1],"day"),round(TLIM[2],"day"),by="24 hours"),format = "%m/%d",labels=TRUE,tcl=-0.25,padj=.0,las=1,lwd=2,font=2)
              mtext("day",1,font=FONT,line=1.5)
            }
            if(m==9){
              axis(2,seq(250,750,250),tck=-.1,label=TRUE,las=2,hadj=.3,lwd=2,font=FONT,line = 0.1,col=NA)
              mtext("height (m)",2,font=FONT,line=1.6)
            }
          }else{
            if(ffdd=="ff"){
              plot(0,0,col=0,xlim=TLIM,ylim=c(0,1000),xaxt="n",yaxt="n",xlab="",ylab="",bty="n")
              #text(TLIM[1]+3600*1,seq(250,750,250), seq(250,750,250),font=2)
              #axis(2,seq(250,750,250),tck=+.1,label=TRUE,las=1,lwd=2,font=2) 
              #mtext("height (m)",2,font=2,line=-6.9,padj=-1,las=2)
            }else{
              par(cex=1)
              image.plot(2,smallplot=SMALLPLOT,horizontal=TRUE,legend.lab=list(ff=expression(paste("Wind speed (m s"^"-1",")")),dd=expression(paste("Wind direction ()")))[[ffdd]],legend.cex=1.3,legend.line=-2.4, breaks=BREAKS,col=COL,legend.only = TRUE,add=TRUE,axis.args=list( at=TICKS,padj=-3.9, tck=1, labels=names(TICKS),font=FONT ))
            }
          }
        }
      }
      if(PRINT) dev.off()
    }
  } 
  
}

# plot case study B contour # review
if(0){
  PRINT=1
  
  WIDTH=8
  HEIGHT=8
  for( fs in 1:2 ){
    for( xCase in 3 ){
      filename = list(a="fig16",b="sup08")[[fs]]
      MODELS = list(a=c(1,7,8,9,-1),b=c(1,3,5,7,-1))[[fs]]
      if(PRINT){ pdf(paste("T:/cosmoPaper/figures/",filename,".pdf",sep=""), width=WIDTH ,height=HEIGHT) }
      #if(PRINT){ png(paste("T:/cosmoPaper/figures/",filename,".png",sep=""),width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
      #if(PRINT){ png(paste("T:/cosmoPaper/lidar/paper/caseStudy",xCase,".png",sep=""),width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
      par(mfrow=c(10,1),omi=c(.01,.5,.01,.01),mai=c(0,0,0,0))
      FONT = 2
      #par(mfrow=c(length(model)*2+2,1),omi=c(0,.4,.2,.4),mai=rep(0.,4))
      for( ffdd in c("ff","dd") ){
        for( m in MODELS ){
          if(m==2){plot(0,0,col=0,xlim=TLIM,ylim=c(0,1000),xaxt="n",yaxt="n"); text(mean(TLIM),800,"C01 crashed",font=2,cex=2)}
          par(cex=1)
          defaultSMALLPLOT=c(.2,.8,.50,.65)
          if(xCase==3){
            # case study B
            TLIM = as.POSIXct(c("2016/01/20 00:00","2016/01/30 00:00"),tz="UTC")
            if(ffdd=="ff"){H=1/100; N=12.5*100+1; M=0; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c(tim.colors(N)[-N],"purple"); TICKS=M+seq(0,N*H*2,H*5*100) }
            if(ffdd=="dd"){H=45/8/100; N=10*100; M=180; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c("purple",tim.colors(N)[c(-1,-N)],"purple"); TICKS=M+seq(0,N*H*2,H*8*100) }
            if(ffdd=="dd"){H=18/100; N=10*100+1; M=0; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=colorRampPalette(c("purple","firebrick1","yellow","deepskyblue","purple"))(N); TICKS=M+seq(0,N*H*2,H*5*100) }
          }
          if(m>0){
            image(range(model[[m]]$meta$time),1:2,(1:2)%*%t(1:2)*0-999,xaxt="n",yaxt="n",xlab="",ylab="",xlim=TLIM,ylim=c(0,1000),breaks=BREAKS,col=COL)
            for( i in which(TLIM[1]<=model[[m]]$meta$time&model[[m]]$meta$time<=TLIM[2]) ){
              image(model[[m]]$meta$time[c(i,i)]+c(0,median(diff(as.numeric(model[[m]]$meta$time)))),model[[m]]$H[i,],limz(model[[m]][[ffdd]][c(i,NA),],ZLIM),breaks=BREAKS,col=COL,add=TRUE)
            }
            index = which(TLIM[1]<=model[[1]]$meta$time&model[[1]]$meta$time<=TLIM[2])
            contour(model[[1]]$meta$time[index],model[[1]]$H[index[1],],is.na(model[[1]][[ffdd]][index,]),level=0.5,drawlabels=FALSE,add=TRUE)
            box()
            text( TLIM%*%c(.5,.5), 750 , model[[m]]$name ,cex=1.5, font=FONT)
            axis(2,seq(0,1000,250),tck=.1,label=FALSE,lwd=2) 
            axis(4,seq(0,1000,250),tck=.1,label=FALSE,lwd=2) 
            #axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="1 hour"),labels=F,lwd=1,tcl=0.15)
            axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="6 hour"),labels=F,lwd=2,tcl=0.25)
            #axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="1 hour"),labels=F,lwd=1,tcl=0.15)
            axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="6 hour"),labels=F,lwd=2,tcl=0.25)
            if(m==MODELS[4]){
              #axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="6 hours")[-1],format = "%H:%M",labels=TRUE,tcl=-0.25,padj=1,las=1,lwd=2,font=2)
              axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="day")[-c(1,length(seq(TLIM[1],TLIM[2],by="day")))],format = "%d",labels=TRUE,tcl=-0.25,padj=-1.5,las=1,lwd=2,font=FONT)
              #axis.POSIXct(side=3,at = seq(round(TLIM[1],"day"),round(TLIM[2],"day"),by="24 hours"),format = "%m/%d",labels=TRUE,tcl=-0.25,padj=.0,las=1,lwd=2,font=2)
              mtext("day",1,font=FONT,line=0.3,adj = .45)
              axis(2,seq(250,750,250),tck=-.1,label=TRUE,las=2,hadj=.3,lwd=2,font=FONT,line = 0.1,col=NA)
              mtext("height (m)",2,font=FONT,line=1.6)
            }
          }else{
            plot(0,0,col=0,xlim=TLIM,ylim=c(0,1000),xaxt="n",yaxt="n",xlab="",ylab="",bty="n")
            par(cex=1)
            image.plot(2,smallplot=SMALLPLOT,horizontal=TRUE,legend.lab=list(ff=expression(paste("Wind speed (m s"^"-1",")")),dd=expression(paste("Wind direction ()")))[[ffdd]],legend.cex=1.3,legend.line=1.0, breaks=BREAKS,col=COL,legend.only = TRUE,add=TRUE,axis.args=list( at=TICKS,padj=-1.8, tck=1, labels=names(TICKS),font=FONT ))
          }
        }
      }
      if(PRINT) dev.off()
    }
  } 
  
}

# plot case study A contour # review
if(0){
  PRINT=1
  
  WIDTH=9
  HEIGHT=6
  
  for( xCase in 1 ){
    if(PRINT){ pdf(paste("T:/cosmoPaper/figures/fig15.pdf",sep=""), width=WIDTH ,height=HEIGHT) }
    #if(PRINT){ png(paste("T:/cosmoPaper/figures/fig15.png",sep=""),width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
    #if(PRINT){ png(paste("T:/cosmoPaper/lidar/paper/caseStudy",xCase,".png",sep=""),width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
    par(mfrow=c(length(model)-2,2),omi=c(.01,.5,.01,.01))
    FONT = 2
    #par(mfrow=c(length(model)*2+2,1),omi=c(0,.4,.2,.4),mai=rep(0.,4))
    for( m in c(1,3,5,7,8,9,-1) ){
      for( ffdd in c("ff","dd") ){
        if(ffdd=="ff"){ par(mai=c(0,0,0,0.05)) }
        if(ffdd=="dd"){ par(mai=c(0,0.05,0,0)) }
        if(m==2){plot(0,0,col=0,xlim=TLIM,ylim=c(0,1000),xaxt="n",yaxt="n"); text(mean(TLIM),800,"C01 crashed",font=2,cex=2)}
        par(cex=1)
        #for( ffdd in c("ff","dd") ){
        defaultSMALLPLOT=c(.1,.9,.45,.60)
        if(xCase==1){
          # case study A
          TLIM = as.POSIXct(c("2016/01/16 18:00","2016/01/17 24:00"),tz="UTC")
          if(ffdd=="ff"){ H=1/100; N=8*100+1; M=0; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c(tim.colors(N)[-N],"purple"); TICKS=M+seq(0,N*H*2,H*4*100) }
          if(ffdd=="dd"){ H=45/8/100; N=12*100+1; M=45; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c("purple",tim.colors(N)[c(-1,-N)],"purple"); TICKS=M+seq(0,N*H*2,H*8*100) }
        }
        if(xCase==2){
          # case study C
          TLIM = as.POSIXct(c("2015/12/30 12:00","2016/01/02 12:00"),tz="UTC")
          if(ffdd=="ff"){ H=1/100; N=10*100; M=0; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c(tim.colors(N)[-N],"purple"); TICKS=M+seq(0,N*H*2,H*2*100) }
          if(ffdd=="dd"){ H=5/100; N=40*30; M=135; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c("purple",tim.colors(N)[c(-1,-N)],"purple"); TICKS=M+seq(0,N*H*2,H*9*100) }
        }
        
        if(m>0){
          image(range(model[[m]]$meta$time),1:2,(1:2)%*%t(1:2)*0-999,xaxt="n",yaxt="n",xlab="",ylab="",xlim=TLIM,ylim=c(0,1000),breaks=BREAKS,col=COL)
          for( i in which(TLIM[1]<=model[[m]]$meta$time&model[[m]]$meta$time<=TLIM[2]) ){
            image(model[[m]]$meta$time[c(i,i)]+c(0,median(diff(as.numeric(model[[m]]$meta$time)))),model[[m]]$H[i,],limz(model[[m]][[ffdd]][c(i,NA),],ZLIM),breaks=BREAKS,col=COL,add=TRUE)
          }
          index = which(TLIM[1]<=model[[1]]$meta$time&model[[1]]$meta$time<=TLIM[2])
          contour(model[[1]]$meta$time[index],model[[1]]$H[i,],is.na(model[[1]][[ffdd]][index,]),level=0.5,drawlabels=FALSE,add=TRUE)
          box()
          text( TLIM%*%c(.8,.2), 500 , model[[m]]$name ,cex=1.3, font=FONT)
          axis(2,seq(0,1000,250),tck=.1,label=FALSE,lwd=2) 
          axis(4,seq(0,1000,250),tck=.1,label=FALSE,lwd=2) 
          axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="1 hour"),labels=F,lwd=1,tcl=0.15)
          axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="6 hour"),labels=F,lwd=2,tcl=0.25)
          axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="1 hour"),labels=F,lwd=1,tcl=0.15)
          axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="6 hour"),labels=F,lwd=2,tcl=0.25)
          if(m==9){
            axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2]-1,by="6 hours")[-1],format = "%H:%M",labels=TRUE,tcl=-0.25,padj=-1.5,las=1,lwd=2,font=FONT)
            #axis.POSIXct(side=1,at = seq(round(TLIM[1],"day"),round(TLIM[2],"day"),by="24 hours"),format = "%m/%d",labels=TRUE,tcl=-0.25,padj=.0,las=1,lwd=2,font=2)
            mtext("time",1,font=FONT,line=0.5)
          }
          if(m==9){
            if(ffdd=="ff"){
              axis(2,seq(250,750,250),tck=-.1,label=TRUE,las=2,hadj=.3,lwd=2,font=FONT,line = 0.1,col=NA)
              mtext("height (m)",2,font=FONT,line=1.6)
            }
          }
        }else{
          plot(0,0,col=0,xlim=TLIM,ylim=c(0,1000),xaxt="n",yaxt="n",xlab="",ylab="",bty="n")
          # if(ffdd=="ff"){
          #   axis(4,seq(250,750,250),tck=-.1,label=TRUE,las=2,hadj=.3,lwd=2,font=2) 
          #   mtext("height (m)",4,font=2,line=1.9)
          # }
          image.plot(2,smallplot=SMALLPLOT,horizontal=TRUE,legend.lab=list(ff=expression(paste("Wind speed (m s"^"-1",")")),dd=expression(paste("Wind direction ()")))[[ffdd]],legend.cex=1.2,legend.line=1.1, breaks=BREAKS,col=COL,legend.only = TRUE,add=TRUE,axis.args=list( at=TICKS,padj=-1.8, tck=1, labels=names(TICKS),font=FONT ))
        }
      }
    }
    if(PRINT) dev.off()
  }
  
}

# plot case study B dotted
if(0){
  PRINT=1
  
  WIDTH=8
  HEIGHT=8
  for( fs in 1:2 ){
    for( xCase in 3 ){
      filename = list(a="fig16",b="sup08")[[fs]]
      MODELS = list(a=c(1,7,8,9,-1),b=c(1,3,5,7,-1))[[fs]]
      if(PRINT){ pdf(paste("T:/cosmoPaper/figures/",filename,".pdf",sep=""), width=WIDTH ,height=HEIGHT) }
      #if(PRINT){ png(paste("T:/cosmoPaper/figures/",filename,".png",sep=""),width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
      #if(PRINT){ png(paste("T:/cosmoPaper/lidar/paper/caseStudy",xCase,".png",sep=""),width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
      par(mfrow=c(10,1),omi=c(.01,.5,.01,.01),mai=c(0,0,0,0))
      FONT = 2
      #par(mfrow=c(length(model)*2+2,1),omi=c(0,.4,.2,.4),mai=rep(0.,4))
      for( ffdd in c("ff","dd") ){
        for( m in MODELS ){
          if(m==2){plot(0,0,col=0,xlim=TLIM,ylim=c(0,1000),xaxt="n",yaxt="n"); text(mean(TLIM),800,"C01 crashed",font=2,cex=2)}
          par(cex=1)
          defaultSMALLPLOT=c(.2,.8,.50,.65)
          if(xCase==3){
            # case study B
            TLIM = as.POSIXct(c("2016/01/20 00:00","2016/01/30 00:00"),tz="UTC")
            if(ffdd=="ff"){H=1/100; N=12.5*100+1; M=0; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c(tim.colors(N)[-N],"purple"); TICKS=M+seq(0,N*H*2,H*5*100) }
            if(ffdd=="dd"){H=45/8/100; N=10*100; M=180; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c("purple",tim.colors(N)[c(-1,-N)],"purple"); TICKS=M+seq(0,N*H*2,H*8*100) }
            if(ffdd=="dd"){H=18/100; N=10*100+1; M=0; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=colorRampPalette(c("purple","firebrick1","yellow","deepskyblue","purple"))(N); TICKS=M+seq(0,N*H*2,H*5*100) }
          }
          if(m>0){
            image(range(model[[m]]$meta$time),1:2,(1:2)%*%t(1:2)*0-999,xaxt="n",yaxt="n",xlab="",ylab="",xlim=TLIM,ylim=c(0,1000),breaks=BREAKS,col=COL)
            for( i in which(TLIM[1]<=model[[m]]$meta$time&model[[m]]$meta$time<=TLIM[2]) ){
              image(model[[m]]$meta$time[c(i,i)]+c(0,median(diff(as.numeric(model[[m]]$meta$time)))),model[[m]]$H[i,],limz(model[[m]][[ffdd]][c(i,NA),],ZLIM),breaks=BREAKS,col=COL,add=TRUE)
              if(m>1){
                index1 = which(model[[m]]$meta$time[i]==model[[1]]$meta$time)
                if(length(index1)){
                  index2 = which(is.na(model[[1]][[ffdd]][index1,]))
                }else{
                  index2 = 1:length(model[[1]][[ffdd]][index1,])
                }
                if(length(index2))
                  points(rep(model[[m]]$meta$time[i],length(index2)),model[[m]]$H[i,index2],pch=19)
              }
            }
            box()
            text( TLIM%*%c(.5,.5), 750 , model[[m]]$name ,cex=1.5, font=FONT)
            axis(2,seq(0,1000,250),tck=.1,label=FALSE,lwd=2) 
            axis(4,seq(0,1000,250),tck=.1,label=FALSE,lwd=2) 
            #axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="1 hour"),labels=F,lwd=1,tcl=0.15)
            axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="6 hour"),labels=F,lwd=2,tcl=0.25)
            #axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="1 hour"),labels=F,lwd=1,tcl=0.15)
            axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="6 hour"),labels=F,lwd=2,tcl=0.25)
            if(m==MODELS[4]){
              #axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="6 hours")[-1],format = "%H:%M",labels=TRUE,tcl=-0.25,padj=1,las=1,lwd=2,font=2)
              axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="day")[-c(1,length(seq(TLIM[1],TLIM[2],by="day")))],format = "%d",labels=TRUE,tcl=-0.25,padj=-1.5,las=1,lwd=2,font=FONT)
              #axis.POSIXct(side=3,at = seq(round(TLIM[1],"day"),round(TLIM[2],"day"),by="24 hours"),format = "%m/%d",labels=TRUE,tcl=-0.25,padj=.0,las=1,lwd=2,font=2)
              mtext("day",1,font=FONT,line=0.3,adj = .45)
              axis(2,seq(250,750,250),tck=-.1,label=TRUE,las=2,hadj=.3,lwd=2,font=FONT,line = 0.1,col=NA)
              mtext("height (m)",2,font=FONT,line=1.6)
            }
          }else{
            plot(0,0,col=0,xlim=TLIM,ylim=c(0,1000),xaxt="n",yaxt="n",xlab="",ylab="",bty="n")
            par(cex=1)
            image.plot(2,smallplot=SMALLPLOT,horizontal=TRUE,legend.lab=list(ff=expression(paste("Wind speed (m s"^"-1",")")),dd=expression(paste("Wind direction ()")))[[ffdd]],legend.cex=1.3,legend.line=1.0, breaks=BREAKS,col=COL,legend.only = TRUE,add=TRUE,axis.args=list( at=TICKS,padj=-1.8, tck=1, labels=names(TICKS),font=FONT ))
          }
        }
      }
      if(PRINT) dev.off()
    }
  } 
  
}

# plot case study A dotted
if(0){
  PRINT=1
  
  WIDTH=9
  HEIGHT=6
  
  for( xCase in 1 ){
    if(PRINT){ pdf(paste("T:/cosmoPaper/figures/fig15.pdf",sep=""), width=WIDTH ,height=HEIGHT) }
    #if(PRINT){ png(paste("T:/cosmoPaper/figures/fig15.png",sep=""),width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
    #if(PRINT){ png(paste("T:/cosmoPaper/lidar/paper/caseStudy",xCase,".png",sep=""),width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
    par(mfrow=c(length(model)-2,2),omi=c(.01,.5,.01,.01))
    FONT = 2
    #par(mfrow=c(length(model)*2+2,1),omi=c(0,.4,.2,.4),mai=rep(0.,4))
    for( m in c(1,3,5,7,8,9,-1) ){
      for( ffdd in c("ff","dd") ){
        if(ffdd=="ff"){ par(mai=c(0,0,0,0.05)) }
        if(ffdd=="dd"){ par(mai=c(0,0.05,0,0)) }
        if(m==2){plot(0,0,col=0,xlim=TLIM,ylim=c(0,1000),xaxt="n",yaxt="n"); text(mean(TLIM),800,"C01 crashed",font=2,cex=2)}
        par(cex=1)
        #for( ffdd in c("ff","dd") ){
        defaultSMALLPLOT=c(.1,.9,.45,.60)
        if(xCase==1){
          # case study A
          TLIM = as.POSIXct(c("2016/01/16 18:00","2016/01/17 24:00"),tz="UTC")
          if(ffdd=="ff"){ H=1/100; N=8*100+1; M=0; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c(tim.colors(N)[-N],"purple"); TICKS=M+seq(0,N*H*2,H*4*100) }
          if(ffdd=="dd"){ H=45/8/100; N=12*100+1; M=45; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c("purple",tim.colors(N)[c(-1,-N)],"purple"); TICKS=M+seq(0,N*H*2,H*8*100) }
        }
        if(xCase==2){
          # case study C
          TLIM = as.POSIXct(c("2015/12/30 12:00","2016/01/02 12:00"),tz="UTC")
          if(ffdd=="ff"){ H=1/100; N=10*100; M=0; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c(tim.colors(N)[-N],"purple"); TICKS=M+seq(0,N*H*2,H*2*100) }
          if(ffdd=="dd"){ H=5/100; N=40*30; M=135; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c("purple",tim.colors(N)[c(-1,-N)],"purple"); TICKS=M+seq(0,N*H*2,H*9*100) }
        }
        
        if(m>0){
          image(range(model[[m]]$meta$time),1:2,(1:2)%*%t(1:2)*0-999,xaxt="n",yaxt="n",xlab="",ylab="",xlim=TLIM,ylim=c(0,1000),breaks=BREAKS,col=COL)
          for( i in which(TLIM[1]<=model[[m]]$meta$time&model[[m]]$meta$time<=TLIM[2]) ){
            image(model[[m]]$meta$time[c(i,i)]+c(0,median(diff(as.numeric(model[[m]]$meta$time)))),model[[m]]$H[i,],limz(model[[m]][[ffdd]][c(i,NA),],ZLIM),breaks=BREAKS,col=COL,add=TRUE)
            if(m>1){
              index1 = which(model[[m]]$meta$time[i]==model[[1]]$meta$time)
              if(length(index1)){
                index2 = which(is.na(model[[1]][[ffdd]][index1,]))
              }else{
                index2 = 1:length(model[[1]][[ffdd]][index1,])
              }
              if(length(index2))
                points(rep(model[[m]]$meta$time[i],length(index2)),model[[m]]$H[i,index2],pch=20,cex=.5)
            }
          }
          box()
          text( TLIM%*%c(.8,.2), 750 , model[[m]]$name ,cex=1.3, font=FONT)
          axis(2,seq(0,1000,250),tck=.1,label=FALSE,lwd=2) 
          axis(4,seq(0,1000,250),tck=.1,label=FALSE,lwd=2) 
          axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="1 hour"),labels=F,lwd=1,tcl=0.15)
          axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="6 hour"),labels=F,lwd=2,tcl=0.25)
          axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="1 hour"),labels=F,lwd=1,tcl=0.15)
          axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="6 hour"),labels=F,lwd=2,tcl=0.25)
          if(m==9){
            axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2]-1,by="6 hours")[-1],format = "%H:%M",labels=TRUE,tcl=-0.25,padj=-1.5,las=1,lwd=2,font=FONT)
            #axis.POSIXct(side=1,at = seq(round(TLIM[1],"day"),round(TLIM[2],"day"),by="24 hours"),format = "%m/%d",labels=TRUE,tcl=-0.25,padj=.0,las=1,lwd=2,font=2)
            mtext("time",1,font=FONT,line=0.5)
          }
          if(m==9){
            if(ffdd=="ff"){
              axis(2,seq(250,750,250),tck=-.1,label=TRUE,las=2,hadj=.3,lwd=2,font=FONT,line = 0.1,col=NA)
              mtext("height (m)",2,font=FONT,line=1.6)
            }
          }
        }else{
          plot(0,0,col=0,xlim=TLIM,ylim=c(0,1000),xaxt="n",yaxt="n",xlab="",ylab="",bty="n")
          # if(ffdd=="ff"){
          #   axis(4,seq(250,750,250),tck=-.1,label=TRUE,las=2,hadj=.3,lwd=2,font=2) 
          #   mtext("height (m)",4,font=2,line=1.9)
          # }
          image.plot(2,smallplot=SMALLPLOT,horizontal=TRUE,legend.lab=list(ff=expression(paste("Wind speed (m s"^"-1",")")),dd=expression(paste("Wind direction ()")))[[ffdd]],legend.cex=1.2,legend.line=1.1, breaks=BREAKS,col=COL,legend.only = TRUE,add=TRUE,axis.args=list( at=TICKS,padj=-1.8, tck=1, labels=names(TICKS),font=FONT ))
        }
      }
    }
    if(PRINT) dev.off()
  }
  
}

# plot case study B shaded
if(0){
  PRINT=1
  
  WIDTH=8
  HEIGHT=8
  for( fs in 1:2 ){
    for( xCase in 3 ){
      filename = list(a="fig16",b="sup08")[[fs]]
      MODELS = list(a=c(1,7,8,9,-1),b=c(1,3,5,7,-1))[[fs]]
      if(PRINT){ pdf(paste("T:/cosmoPaper/figures/",filename,".pdf",sep=""), width=WIDTH ,height=HEIGHT) }
      #if(PRINT){ png(paste("T:/cosmoPaper/figures/",filename,".png",sep=""),width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
      #if(PRINT){ png(paste("T:/cosmoPaper/lidar/paper/caseStudy",xCase,".png",sep=""),width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
      par(mfrow=c(10,1),omi=c(.01,.5,.01,.01),mai=c(0,0,0,0))
      FONT = 2
      #par(mfrow=c(length(model)*2+2,1),omi=c(0,.4,.2,.4),mai=rep(0.,4))
      for( ffdd in c("ff","dd") ){
        for( m in MODELS ){
          if(m==2){plot(0,0,col=0,xlim=TLIM,ylim=c(0,1000),xaxt="n",yaxt="n"); text(mean(TLIM),800,"C01 crashed",font=2,cex=2)}
          par(cex=1)
          defaultSMALLPLOT=c(.2,.8,.50,.65)
          if(xCase==3){
            # case study B
            TLIM = as.POSIXct(c("2016/01/20 00:00","2016/01/30 00:00"),tz="UTC")
            if(ffdd=="ff"){H=1/100; N=12.5*100+1; M=0; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c(tim.colors(N)[-N],"purple"); TICKS=M+seq(0,N*H*2,H*5*100) }
            if(ffdd=="dd"){H=45/8/100; N=10*100; M=180; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c("purple",tim.colors(N)[c(-1,-N)],"purple"); TICKS=M+seq(0,N*H*2,H*8*100) }
            if(ffdd=="dd"){H=18/100; N=10*100+1; M=0; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=colorRampPalette(c("purple","firebrick1","yellow","deepskyblue","purple"))(N); TICKS=M+seq(0,N*H*2,H*5*100) }
          }
          shadedCOL = "#FFFFFFA0"
          #  COL = c("purple",tim.colors(diff(ZLIM)/(10^(ffdd=="dd"))-1),"purple")
          if(m>0){
            image(range(model[[m]]$meta$time),1:2,(1:2)%*%t(1:2)*0-999,xaxt="n",yaxt="n",xlab="",ylab="",xlim=TLIM,ylim=c(0,1000),breaks=BREAKS,col=COL)
            for( i in which(TLIM[1]<=model[[m]]$meta$time&model[[m]]$meta$time<=TLIM[2]) ){
              image(model[[m]]$meta$time[c(i,i)]+c(0,median(diff(as.numeric(model[[m]]$meta$time)))),model[[m]]$H[i,],limz(model[[m]][[ffdd]][c(i,NA),],ZLIM),breaks=BREAKS,col=COL,add=TRUE)
              if(m>1){
                index1 = which(model[[m]]$meta$time[i]==model[[1]]$meta$time)
                if(length(index)){
                  index2 = which(is.na(model[[1]][[ffdd]][index1,]))
                  if(length(index2)){
                    shadedZ = model[[m]][[ffdd]][c(i,NA),]*NA
                    shadedZ[1,index2] = 1
                  }
                }else{
                  shadedZ[1,] = 1
                }
                if(length(index)==0|length(index2))
                  image(model[[m]]$meta$time[c(i,i)]+c(0,median(diff(as.numeric(model[[m]]$meta$time)))),model[[m]]$H[i,][],shadedZ,col=shadedCOL,add=TRUE)
              }
            }
            box()
            text( TLIM%*%c(.5,.5), 750 , model[[m]]$name ,cex=1.5, font=FONT)
            axis(2,seq(0,1000,250),tck=.1,label=FALSE,lwd=2) 
            axis(4,seq(0,1000,250),tck=.1,label=FALSE,lwd=2) 
            #axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="1 hour"),labels=F,lwd=1,tcl=0.15)
            axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="6 hour"),labels=F,lwd=2,tcl=0.25)
            #axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="1 hour"),labels=F,lwd=1,tcl=0.15)
            axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="6 hour"),labels=F,lwd=2,tcl=0.25)
            if(m==MODELS[4]){
              #axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="6 hours")[-1],format = "%H:%M",labels=TRUE,tcl=-0.25,padj=1,las=1,lwd=2,font=2)
              axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="day")[-c(1,length(seq(TLIM[1],TLIM[2],by="day")))],format = "%d",labels=TRUE,tcl=-0.25,padj=-1.5,las=1,lwd=2,font=FONT)
              #axis.POSIXct(side=3,at = seq(round(TLIM[1],"day"),round(TLIM[2],"day"),by="24 hours"),format = "%m/%d",labels=TRUE,tcl=-0.25,padj=.0,las=1,lwd=2,font=2)
              mtext("day",1,font=FONT,line=0.3,adj = .45)
              axis(2,seq(250,750,250),tck=-.1,label=TRUE,las=2,hadj=.3,lwd=2,font=FONT,line = 0.1,col=NA)
              mtext("height (m)",2,font=FONT,line=1.6)
            }
          }else{
            plot(0,0,col=0,xlim=TLIM,ylim=c(0,1000),xaxt="n",yaxt="n",xlab="",ylab="",bty="n")
            par(cex=1)
            image.plot(2,smallplot=SMALLPLOT,horizontal=TRUE,legend.lab=list(ff=expression(paste("Wind speed (m s"^"-1",")")),dd=expression(paste("Wind direction ()")))[[ffdd]],legend.cex=1.3,legend.line=1.0, breaks=BREAKS,col=COL,legend.only = TRUE,add=TRUE,axis.args=list( at=TICKS,padj=-1.8, tck=1, labels=names(TICKS),font=FONT ))
          }
        }
      }
      if(PRINT) dev.off()
    }
  } 
  
}

# plot case study A shaded
if(0){
  PRINT=0
  
  WIDTH=9
  HEIGHT=6
  
  for( xCase in 1 ){
    if(PRINT){ pdf(paste("T:/cosmoPaper/figures/fig15.pdf",sep=""), width=WIDTH ,height=HEIGHT) }
    #if(PRINT){ png(paste("T:/cosmoPaper/figures/fig15.png",sep=""),width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
    #if(PRINT){ png(paste("T:/cosmoPaper/lidar/paper/caseStudy",xCase,".png",sep=""),width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
    par(mfrow=c(length(model)-2,2),omi=c(.01,.5,.01,.01))
    FONT = 2
    #par(mfrow=c(length(model)*2+2,1),omi=c(0,.4,.2,.4),mai=rep(0.,4))
    for( m in c(1,3,5,7,8,9,-1) ){
      for( ffdd in c("ff","dd") ){
        if(ffdd=="ff"){ par(mai=c(0,0,0,0.05)) }
        if(ffdd=="dd"){ par(mai=c(0,0.05,0,0)) }
        if(m==2){plot(0,0,col=0,xlim=TLIM,ylim=c(0,1000),xaxt="n",yaxt="n"); text(mean(TLIM),800,"C01 crashed",font=2,cex=2)}
        par(cex=1)
        #for( ffdd in c("ff","dd") ){
        defaultSMALLPLOT=c(.1,.9,.45,.60)
        if(xCase==1){
          # case study A
          TLIM = as.POSIXct(c("2016/01/16 18:00","2016/01/17 24:00"),tz="UTC")
          if(ffdd=="ff"){ H=1/100; N=8*100+1; M=0; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c(tim.colors(N)[-N],"purple"); TICKS=M+seq(0,N*H*2,H*4*100) }
          if(ffdd=="dd"){ H=45/8/100; N=12*100+1; M=45; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c("purple",tim.colors(N)[c(-1,-N)],"purple"); TICKS=M+seq(0,N*H*2,H*8*100) }
        }
        if(xCase==2){
          # case study C
          TLIM = as.POSIXct(c("2015/12/30 12:00","2016/01/02 12:00"),tz="UTC")
          if(ffdd=="ff"){ H=1/100; N=10*100; M=0; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c(tim.colors(N)[-N],"purple"); TICKS=M+seq(0,N*H*2,H*2*100) }
          if(ffdd=="dd"){ H=5/100; N=40*30; M=135; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=c("purple",tim.colors(N)[c(-1,-N)],"purple"); TICKS=M+seq(0,N*H*2,H*9*100) }
        }
        shadedCOL = "#FFFFFFA0"
        #  COL = c("purple",tim.colors(diff(ZLIM)/(10^(ffdd=="dd"))-1),"purple")
        if(m>0){
          image(range(model[[m]]$meta$time),1:2,(1:2)%*%t(1:2)*0-999,xaxt="n",yaxt="n",xlab="",ylab="",xlim=TLIM,ylim=c(0,1000),breaks=BREAKS,col=COL)
          for( i in which(TLIM[1]<=model[[m]]$meta$time&model[[m]]$meta$time<=TLIM[2]) ){
            image(model[[m]]$meta$time[c(i,i)]+c(0,median(diff(as.numeric(model[[m]]$meta$time)))),model[[m]]$H[i,],limz(model[[m]][[ffdd]][c(i,NA),],ZLIM),breaks=BREAKS,col=COL,add=TRUE)
            if(m>1){
              index1 = which(model[[m]]$meta$time[i]==model[[1]]$meta$time)
              if(length(index1)){
                index2 = which(is.na(model[[1]][[ffdd]][index1,]))
                if(length(index2)){
                  shadedZ = model[[m]][[ffdd]][c(i,NA),]*NA
                  shadedZ[1,index2] = 1
                }
              }else{
                shadedZ[1,] = 1
              }
              if(length(index1)==0|length(index2))
              image(model[[m]]$meta$time[c(i,i)]+c(0,median(diff(as.numeric(model[[m]]$meta$time)))),model[[m]]$H[i,][],shadedZ,col=shadedCOL,add=TRUE)
            }
          }
          box()
          text( TLIM%*%c(.8,.2), 750 , model[[m]]$name ,cex=1.3, font=FONT)
          axis(2,seq(0,1000,250),tck=.1,label=FALSE,lwd=2) 
          axis(4,seq(0,1000,250),tck=.1,label=FALSE,lwd=2) 
          axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="1 hour"),labels=F,lwd=1,tcl=0.15)
          axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="6 hour"),labels=F,lwd=2,tcl=0.25)
          axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="1 hour"),labels=F,lwd=1,tcl=0.15)
          axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="6 hour"),labels=F,lwd=2,tcl=0.25)
          if(m==9){
            axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2]-1,by="6 hours")[-1],format = "%H:%M",labels=TRUE,tcl=-0.25,padj=-1.5,las=1,lwd=2,font=FONT)
            #axis.POSIXct(side=1,at = seq(round(TLIM[1],"day"),round(TLIM[2],"day"),by="24 hours"),format = "%m/%d",labels=TRUE,tcl=-0.25,padj=.0,las=1,lwd=2,font=2)
            mtext("time",1,font=FONT,line=0.5)
          }
          if(m==9){
            if(ffdd=="ff"){
              axis(2,seq(250,750,250),tck=-.1,label=TRUE,las=2,hadj=.3,lwd=2,font=FONT,line = 0.1,col=NA)
              mtext("height (m)",2,font=FONT,line=1.6)
            }
          }
        }else{
          plot(0,0,col=0,xlim=TLIM,ylim=c(0,1000),xaxt="n",yaxt="n",xlab="",ylab="",bty="n")
          # if(ffdd=="ff"){
          #   axis(4,seq(250,750,250),tck=-.1,label=TRUE,las=2,hadj=.3,lwd=2,font=2) 
          #   mtext("height (m)",4,font=2,line=1.9)
          # }
          image.plot(2,smallplot=SMALLPLOT,horizontal=TRUE,legend.lab=list(ff=expression(paste("Wind speed (m s"^"-1",")")),dd=expression(paste("Wind direction ()")))[[ffdd]],legend.cex=1.2,legend.line=1.1, breaks=BREAKS,col=COL,legend.only = TRUE,add=TRUE,axis.args=list( at=TICKS,padj=-1.8, tck=1, labels=names(TICKS),font=FONT ))
        }
      }
    }
    if(PRINT) dev.off()
  }
  
}

# plot old
if(0){
  PRINT=1
  if(PRINT) png("T:/cosmoPaper/lidar/test.png",height=900,width=900,res=100)
  par(mfrow=c(length(model)+1+1,2),omi=rep(.01,4),mai=c(0,0,0,0))
  #par(mfrow=c(length(model)*2+2,1),omi=c(0,.4,.2,.4),mai=rep(0.,4))
  for( ffdd in c("ff","dd") ){
    for( m in c(1,-1,2:length(model)) ){
      if(m==2){plot(0,0,col=0,xlim=TLIM,ylim=c(0,1000),xaxt="n",yaxt="n"); text(mean(TLIM),800,"C01 crashed",font=2,cex=2)}
      par(cex=1)
      #for( ffdd in c("ff","dd") ){
      defaultSMALLPLOT=c(.2,.95,.45,.60)
      if(ffdd=="ff"){ H=1; N=8; M=0; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=tim.colors(N); TICKS=M+seq(0,N*H*2,H*4) }
      if(ffdd=="ff"){ H=1/100; N=10*100; M=0; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=tim.colors(N); TICKS=M+seq(0,N*H*2,H*2*100) }
      if(ffdd=="dd"){ H=45/8; N=32; M=0; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=tim.colors(N); TICKS=M+seq(0,N*H*2,H*8) }
      if(ffdd=="dd"){ H=5/100; N=60*30; M=80; SMALLPLOT=defaultSMALLPLOT; ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=tim.colors(N); TICKS=M+seq(0,N*H*2,H*2*100) }
      #  COL = c("purple",tim.colors(diff(ZLIM)/(10^(ffdd=="dd"))-1),"purple")
      if(m>0){
        image(range(model[[m]]$meta$time),1:2,(1:2)%*%t(1:2)*0-999,xaxt="n",yaxt="n",xlab="",ylab="",xlim=TLIM,ylim=c(0,1000),breaks=BREAKS,col=COL)
        for( i in seq(model[[m]]$meta$lon) ){
          image(model[[m]]$meta$time[c(i,i)]+c(0,diff(as.numeric(model[[m]]$meta$time))[1]),model[[m]]$H[i,],limz(model[[m]][[ffdd]][c(i,NA),],ZLIM),breaks=BREAKS,col=COL,add=TRUE)
        }
        box()
        text( TLIM%*%c(.8,.2), 750 , model[[m]]$name ,cex=1.5, font=2)
        axis(2,seq(0,1000,250),tck=.1,label=FALSE,lwd=2) 
        axis(4,seq(0,1000,250),tck=.1,label=FALSE,lwd=2) 
        axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="1 hour"),labels=F,lwd=1,tcl=0.15)
        axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="6 hour"),labels=F,lwd=2,tcl=0.25)
        axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="1 hour"),labels=F,lwd=1,tcl=0.15)
        axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="6 hour"),labels=F,lwd=2,tcl=0.25)
        if(m==3){
          axis.POSIXct(side=3,at = seq(TLIM[1],TLIM[2],by="6 hours")[-1],format = "%H:%M",labels=TRUE,tcl=-0.25,padj=1,las=1,lwd=2,font=2)
          axis.POSIXct(side=3,at = seq(round(TLIM[1],"day"),round(TLIM[2],"day"),by="24 hours"),format = "%m/%d",labels=TRUE,tcl=-0.25,padj=.0,las=1,lwd=2,font=2)
          mtext("time",3,font=2,line=2)
        }
        if(m==1){
          axis(4,seq(250,750,250),tck=-.1,label=TRUE,las=2,hadj=.3,lwd=2,font=2) 
          mtext("height (m)",4,font=2,line=1.9)
        }
      }else{
        plot(0,0,col=0,xaxt="n",yaxt="n",xlab="",ylab="",bty="n")
        image.plot(2,smallplot=SMALLPLOT,horizontal=TRUE,legend.lab=list(ff="Wind speed (m/s)",dd="Wind direction ()")[[ffdd]],legend.cex=1.3,legend.line=-2.4, breaks=BREAKS,col=COL,legend.only = TRUE,add=TRUE,axis.args=list( at=TICKS,padj=-3.9, tck=1, labels=names(TICKS),font=2 ))
      }
    }
  }
  if(PRINT) dev.off()
}

# plot overview
if(0){
  alldays = diff(as.numeric(range(TLIM)))/3600/24
  pdf("R:/zentek/scr/frosn/overview.pdf",height=16,width=alldays/10*12)
  par(mfrow=c(length(model)*2+2,1),omi=c(0,.4,.2,.4),mai=rep(0.,4))
  for( ffdd in c("ff","dd") ){
    for( m in c(seq(model),-1) ){
      #for( ffdd in c("ff","dd") ){
      par(cex=1.4)
      if(ffdd=="ff"){ H=1; N=10; M=0; SMALLPLOT=c(.4,.5,.4,.5); ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=tim.colors(N); TICKS=M+seq(0,N*H*2,H*4) }
      if(ffdd=="dd"){ H=45/8; N=32; M=0; SMALLPLOT=c(.4,.5,.4,.5); ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=tim.colors(N); TICKS=M+seq(0,N*H*2,H*8) }
      #  COL = c("purple",tim.colors(diff(ZLIM)/(10^(ffdd=="dd"))-1),"purple")
      if(m>0){
        image(range(model[[m]]$meta$time),1:2,(1:2)%*%t(1:2)*0-999,xaxt="n",yaxt="n",xlab="",ylab="",xlim=TLIM,ylim=c(0,2000),breaks=BREAKS,col=COL)
        for( i in seq(meta$lon) ){
          image(model[[m]]$meta$time[c(i,i)]+c(0,diff(as.numeric(model[[m]]$meta$time))[1]),model[[m]]$H[i,],limz(model[[m]][[ffdd]][c(i,NA),],ZLIM),breaks=BREAKS,col=COL,add=TRUE)
        }
        lines( model[[1]]$meta$time , apply(model[[1]][["ff"]]*0+model[[1]]$H, 1, max, na.rm=TRUE) )
        lines( model[[1]]$meta$time , apply(model[[1]][["ff"]]*0+model[[1]]$H, 1, min, na.rm=TRUE) )
        #if(ffdd=="dd"){
          text( mean(TLIM), 500 , model[[m]]$name ,cex=2)
          axis(2,seq(250,750,250),las=2) 
        #} 
        axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="1 hour"),labels=F,lwd=1,tcl=0.15)
        axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="6 hour"),labels=F,lwd=2,tcl=0.25)
        if(m==length(model)){
          #axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="12 hours"),format = "%H:%M",labels=TRUE,padj=-.7,las=1,lwd=2)
          axis.POSIXct(side=1,at = seq(round(TLIM[1],"day"),round(TLIM[2],"day"),by="24 hours"),format = "%m/%d",labels=TRUE,padj=.5,las=1,lwd=2)
        } 
      }else{
        plot(0,0,col=0,xaxt="n",yaxt="n",xlab="",ylab="",bty="n")
        image.plot(2,smallplot=SMALLPLOT,horizontal=TRUE,legend.lab=ffdd,legend.cex=2.5,legend.line=-1, breaks=BREAKS,col=COL,legend.only = TRUE,add=TRUE,axis.args=list( at=TICKS, labels=names(TICKS) ))
      }
    }
  }
  dev.off()
}

# plot diff C15TC15
if(0){
  alldays = diff(as.numeric(range(TLIM)))/3600/24
  #pdf("R:/zentek/scr/frosn/oveviewDiffC15T15.pdf",height=8,width=alldays/10*12)
  par(mfrow=c(length(model)*2+2-4,1),omi=c(0,.4,.2,.4),mai=rep(0.,4))
  for( ffdd in c("ff","dd") ){
    for( m in c(seq(model)[3],-1) ){
      par(cex=1.4)
      #for( ffdd in c("ff","dd") ){
      par(cex=1.4)
      if(ffdd=="ff"){ H=1/4; N=9; M=-2; SMALLPLOT=c(.1,.9,.4,.5); ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=tim.colors(N); TICKS=M+seq(0,N*H*2,H*4) }
      if(ffdd=="dd"){ H=5/2; N=9; M=-20; SMALLPLOT=c(.1,.9,.4,.5); ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=tim.colors(N); TICKS=M+seq(0,N*H*2,H*8) }
      #  COL = c("purple",tim.colors(diff(ZLIM)/(10^(ffdd=="dd"))-1),"purple")
      if(m>0){
        image(range(model[[m]]$meta$time),1:2,(1:2)%*%t(1:2)*0-999,xaxt="n",yaxt="n",xlab="",ylab="",xlim=TLIM,ylim=c(0,2000),breaks=BREAKS,col=COL)
        for( i in seq(meta$lon) ){
          image(model[[m]]$meta$time[c(i,i)]+c(0,diff(as.numeric(model[[m]]$meta$time))[1]),model[[m]]$H[i,],limz(model[[m]][[ffdd]][c(i,NA),]-model[[m-1]][[ffdd]][c(i,NA),],ZLIM),breaks=BREAKS,col=COL,add=TRUE)
        }
        diff = model[[m]][[ffdd]]-model[[m-1]][[ffdd]]
        
        abline(h=1000,lwd=10)
        lines(model[[m]]$meta$time, apply(diff, 1, mean)*1000+1000,lwd=3)
        if(ffdd=="dd"){
          text( mean(TLIM), 500 , model[[m]]$name ,cex=2)
          axis(2,seq(250,750,250),las=2) 
        } 
        axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="1 hour"),labels=F,lwd=1,tcl=0.15)
        axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="6 hour"),labels=F,lwd=2,tcl=0.25)
        if(m==length(model)){
          #axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="12 hours"),format = "%H:%M",labels=TRUE,padj=-.7,las=1,lwd=2)
          axis.POSIXct(side=1,at = seq(round(TLIM[1],"day"),round(TLIM[2],"day"),by="24 hours"),format = "%m/%d",labels=TRUE,padj=.5,las=1,lwd=2)
        } 
      }else{
        plot(0,0,col=0,xaxt="n",yaxt="n",xlab="",ylab="",bty="n")
        image.plot(2,smallplot=SMALLPLOT,horizontal=TRUE,legend.lab=ffdd,legend.cex=2.5,legend.line=-1, breaks=BREAKS,col=COL,legend.only = TRUE,add=TRUE,axis.args=list( at=TICKS, labels=names(TICKS) ))
      }
    }
  }
  #dev.off()
}

# plot diff C15TC15
if(0){
  alldays = diff(as.numeric(range(TLIM)))/3600/24
  #pdf("R:/zentek/scr/frosn/oveviewDiffC15T15.pdf",height=8,width=alldays/10*12)
  par(mfrow=c(length(model)*2+2-4,1),omi=c(0,.4,.2,.4),mai=rep(0.,4))
  for( ffdd in c("ff","dd") ){
    for( m in c(seq(model)[3],-1) ){
      par(cex=1.4)
      #for( ffdd in c("ff","dd") ){
      par(cex=1.4)
      if(ffdd=="ff"){ H=1/4; N=9; M=-2; SMALLPLOT=c(.1,.9,.4,.5); ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=tim.colors(N); TICKS=M+seq(0,N*H*2,H*4) }
      if(ffdd=="dd"){ H=5/2; N=9; M=-20; SMALLPLOT=c(.1,.9,.4,.5); ZLIM=M+c(-H,H*N*2-H); BREAKS=seq(ZLIM[1],ZLIM[2],H*2); COL=tim.colors(N); TICKS=M+seq(0,N*H*2,H*8) }
      #  COL = c("purple",tim.colors(diff(ZLIM)/(10^(ffdd=="dd"))-1),"purple")
      if(m>0){
        image(range(model[[m]]$meta$time),1:2,(1:2)%*%t(1:2)*0-999,xaxt="n",yaxt="n",xlab="",ylab="",xlim=TLIM,ylim=c(0,2000),breaks=BREAKS,col=COL)
        for( i in seq(meta$lon) ){
          image(model[[m]]$meta$time[c(i,i)]+c(0,diff(as.numeric(model[[m]]$meta$time))[1]),model[[m]]$H[i,],limz(model[[m]][[ffdd]][c(i,NA),]-model[[m-1]][[ffdd]][c(i,NA),],ZLIM),breaks=BREAKS,col=COL,add=TRUE)
        }
        if(ffdd=="dd"){
          text( mean(TLIM), 500 , model[[m]]$name ,cex=2)
          axis(2,seq(250,750,250),las=2) 
        } 
        axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="1 hour"),labels=F,lwd=1,tcl=0.15)
        axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="6 hour"),labels=F,lwd=2,tcl=0.25)
        if(m==length(model)){
          #axis.POSIXct(side=1,at = seq(TLIM[1],TLIM[2],by="12 hours"),format = "%H:%M",labels=TRUE,padj=-.7,las=1,lwd=2)
          axis.POSIXct(side=1,at = seq(round(TLIM[1],"day"),round(TLIM[2],"day"),by="24 hours"),format = "%m/%d",labels=TRUE,padj=.5,las=1,lwd=2)
        } 
      }else{
        plot(0,0,col=0,xaxt="n",yaxt="n",xlab="",ylab="",bty="n")
        image.plot(2,smallplot=SMALLPLOT,horizontal=TRUE,legend.lab=ffdd,legend.cex=2.5,legend.line=-1, breaks=BREAKS,col=COL,legend.only = TRUE,add=TRUE,axis.args=list( at=TICKS, labels=names(TICKS) ))
      }
    }
  }
  #dev.off()
}
