library(ncdf4)
library(data.table)
library(doParallel) 
library(matlab)
source("/home/sida6serv02/zentek/R/ncdf4Utils_1.0/R/geo2rot.R")
source("/home/sida6serv02/zentek/R/rotateWind.R")

outpath="/home/sida6serv02/zentek/scr/raso/cosmo/rdata"
tic() # 1 hour...
nproc = 20
cl <- makeCluster(nproc, outfile="")
registerDoParallel(cl)

# META DATA
meta = data.table(
  name = c("Amundsen-Scott","Halley","Marambio","Neumayer","Novolazarevskaya","Rothera"),
  lon = c(   0, -26.660, -56.633,  -8.274,  11.832, -68.133),
  lat = c( -90, -75.583, -64.233, -70.674, -70.768, -67.566),
  height = c( 2835, 30, 198, 50, 119, 16),
  ix = numeric(6), iy = numeric(6))

load("/home/sida6serv02/zentek/scr/raso/wyoming/rdata/stations.rData")
raso = stations
###### check for error ######
for( i in 1:length(raso) ){
  if( meta$name[i]!=raso[[i]]$name ){
    print(c("error name",raso[[i]]$name,meta$name[i]))
  }
}

#### MAIN PROGRAM ###
for( RES in c("C0518","C1518") ){
  print(paste("======",RES,"======"))
  if(RES=="C1518"){xRES="C15"}
  if(RES=="C0518"){xRES="C05"}
  file = paste("/data/sida6serv03/uwm/zentek/COSMO_",xRES,"_out01/20040404/out01/lffd2004040318c.nc",sep="")
  
  nc = nc_open(file)
  lon = ncvar_get(nc,"lon")
  lat = ncvar_get(nc,"lat")
  nc_close(nc)
  
  data = list()
  TIME = list()
  for( is in 1:length(raso) ){
    
    # find cosmo grid points nearest to station
    dist = sqrt(((lon-meta$lon[is])*cos(meta$lat[is]/180*pi))^2+(lat-meta$lat[is])^2)
    dist = which(dist == min(dist) , arr.ind = TRUE)
    meta[is]$ix = dist[1]
    meta[is]$iy = dist[2]
    
    # subselect time & station for C05
    time = raso[[is]]$meta$time
    if( RES=="C05" ) { 
      if( meta$ix[is]==500 ){
        data[[is]] = NULL
        print(paste("skipping",meta[is]$name))
        next
      }
      time = time[ 4 <= as.numeric(strftime(time,"%m")) & as.numeric(strftime(time,"%m")) <= 9 ]
    }
    if( RES=="C15" ) { # last run with <= 2015
      time = time[ 2002 <= as.numeric(strftime(time,"%Y")) & as.numeric(strftime(time,"%Y")) <= 2015 ]
    }
	
    print(meta[is]$name)
	TIME[[is]] = time
    data[[is]] = foreach( it=1:length(time) , .packages=c("data.table","ncdf4") ) %dopar% {
      if(it%%100==1){ cat( paste(round(it/length(time)*100),"% ",sep="") ) }
      file = paste("/data/sida6serv03/uwm/zentek/COSMO_",xRES,"_out01/",strftime(time[it],"%Y%m%d"),"/out01/lffd",strftime(time[it]-3600*24,"%Y%m%d"),"18.nc",sep="")
      nc = nc_open(file)
      t = ncvar_get(nc,"T",start=c(meta$ix[is],meta$iy[is],1,1),c(1,1,60,1))
      u = apply(ncvar_get(nc,"U",start=c(meta$ix[is],meta$iy[is],1,1),c(2,1,60,1)),2,mean)
      v = apply(ncvar_get(nc,"V",start=c(meta$ix[is],meta$iy[is],1,1),c(1,2,60,1)),2,mean)
      qv = ncvar_get(nc,"QV",start=c(meta$ix[is],meta$iy[is],1,1),c(1,1,60,1))
      pp = ncvar_get(nc,"PP",start=c(meta$ix[is],meta$iy[is],1,1),c(1,1,60,1))
      nc_close(nc)

      return(data.table(t=t,u=u,v=v,qv=qv,pp=pp))
    }
    print("")
      
  }# is
  dir.create(paste(outpath,RES,sep="/"), showWarnings = FALSE, recursive=TRUE)
  cosmo = list(RES=RES,meta=meta,time=TIME,data=data)
  save(cosmo,file=paste(outpath,RES,"stations.rData",sep="/"))
}# RES

stopCluster(cl)
toc()
