# INIT
if(1){
  library(ncdf4)
  library(data.table)
  library(circular)
  source("T:/cosmoPaper/map/geo2rot.R")
  source("T:/cosmoPaper/map/rotateWind.R")
  
  inpath="T:/cosmoPaper/raso"
  outpath="T:/cosmoPaper/raso/rdata"
  
  # META DATA
  meta = data.table(
    name = c("Amundsen-Scott","Halley","Marambio","Neumayer","Novolazarevskaya","Rothera"),
    lon = c(   0, -26.660, -56.633,  -8.274,  11.832, -68.133),
    lat = c( -90, -75.583, -64.233, -70.674, -70.768, -67.566),
    height = c( 2835, 30, 198, 50, 119, 16),
    ix = numeric(6), iy = numeric(6)
  )
  
  # load data
  load("T:/cosmoPaper/raso/rdata/stationsP.rData")
  
  # make some more data
  pref = h = rlon = rlat = c = list()
  for( RES in c("C05","C15") ){
    
    load(paste(inpath,"/cosmo/rdata/station",RES,".rData",sep=""))
    c[[RES]]=cosmo
    
    file = paste("S:/COSMO_",RES,"_out01/20040404/out01/lffd2004040318c.nc",sep="")
    
    nc = nc_open(file)
    land = ncvar_get(nc,"FR_LAND")
    hhl = ncvar_get(nc,"HHL")
    h[[RES]] = (hhl[,,1:60]+hhl[,,2:61])/2
    vcoord = ncvar_get(nc,"vcoord")
    p0sl = ncatt_get(nc,"vcoord","p0sl")$value
    t0sl = ncatt_get(nc,"vcoord","t0sl")$value
    dt0lp = ncatt_get(nc,"vcoord","dt0lp")$value
    delta_t = ncatt_get(nc,"vcoord","delta_t")$value
    h_scal = ncatt_get(nc,"vcoord","h_scal")$value
    g = 9.80665#9.807
    R = 287.05
    t00 = t0sl - delta_t
    # pref = p0sl * exp( -g*h_scal/ R/t00 * log( (exp(vcoord/h_scal)*t00 + delta_t)/(t00 + delta_t)) )
    pref[[RES]] = p0sl * exp( -g*h_scal/ R/t00 * log( (exp(h[[RES]]/h_scal)*t00 + delta_t)/(t00 + delta_t)) )
    rlon[[RES]] = ncvar_get(nc,"rlon")
    rlat[[RES]] = ncvar_get(nc,"rlat")
    lon = ncvar_get(nc,"lon")
    lat = ncvar_get(nc,"lat")
    nc_close(nc)
  }
  
}

### table
if(0){

  #ID ok
  TABLE = meta[,1:4]
  #name
  #TABLE$name = substr( sub("_"," ",TABLE$name) ,1,5)
  TABLE$name = sub("-"," ",TABLE$name)
  #lon ok
  #lat ok
  #height ok
  #cosmo height
  TABLE$C05height = round(h$C05[,,60][as.matrix(station$data[[1]]$C05$meta[,5:6])]) #- TABLE$height
  TABLE$C05height[TABLE$name%in%c("Amundsen Scott","Novolazarevskaya")] = NA
  TABLE$C15height = round(h$C15[,,60][as.matrix(station$data[[1]]$C15$meta[,5:6])]) #- TABLE$height
  #years
  N=obs=interval=NULL
  par(mfrow=c(2,3))
  for(is in seq(TABLE$name)){
    HOURS = as.numeric(names(sort(table(as.numeric(strftime(station$data[[is]]$time,"%H",tz="UTC"))),decreasing=TRUE)[1]))
    MONTH = 1:12
    TIMES1 = as.numeric(strftime(station$data[[is]]$time,"%m",tz="UTC"))%in%MONTH
    TIMES2 = as.numeric(strftime(station$data[[is]]$time,"%H",tz="UTC"))%in%HOURS
    TIMES3 = as.numeric(strftime(station$data[[is]]$time,"%Y",tz="UTC"))%in%(2002:2016)
    TIMES = TIMES1 & TIMES2 & TIMES3
    DIFF= diff(as.numeric(station$data[[is]]$time[TIMES])) / 3600 /24
    HIST = hist(DIFF,xlim=c(0,20),breaks = c(seq(0,20,1),999999))
    N[is] = (length(which(TIMES)))
    obs[is] = HOURS
    interval[is] = which(cumsum(HIST$counts)/sum(HIST$counts)>0.85)[1]
  }
  #  TABLE$winters=winters
  #  TABLE$summers=summers
  TABLE$obs=obs
  TABLE$interval=interval
  TABLE$N=N
  
  TABLE = TABLE[c(3:6,2,1),]
  row.names(TABLE)<-letters[seq(TABLE$name)]
  write.table(TABLE,file="T:/cosmoPaper/raso/paper/table.txt",quote=FALSE,sep=" & ",eol=" \\\\\n")
  #write.table(TABLE,file="T:/cosmoPaper/raso/paper/table.txt",quote=FALSE,sep="\t",eol="\n")
  
}

# sd P
if(1){
  PRINT = 1
  CEX = 0.8
  #RES = c("w","C1518","C05","C15","C15","T15")
  RES = c("C15","C15","C05")
  NAMES = c("C15 winter","C15 summer","C05 winter")
  COLbias = c("darkorchid1","darkorchid3","darkorchid4")
  COLrmse = c("coral2","coral3","coral4")
  COLcorr = c("seagreen1","seagreen3","seagreen4")
  LWD = c(2,2,2)
  LTY = c(1,2,3)
  VAR = c("t","ff","dd","q","p","g")
  XLIM = list(c(-3,9),c(-3,9),c(-50,150),c(-0.001,0.001),c(-0.1,0.1),c(-20,20))
  ABLINE = list(c(seq(-3,9,3)),seq(-3,9,3),seq(-50,150,50),c(-0.001,0.001),c(-0.1,0.1),c(-20,20))
  for(iv in (1:3)[]){
    WIDTH=5
    HEIGHT=4
    filename = list(t="fig12",ff="fig13",dd="fig14")[[VAR[iv]]]
    if(PRINT){ pdf(paste("T:/cosmoPaper/figures/",filename,".pdf",sep=""),width=WIDTH,height=HEIGHT) }
    #if(PRINT){ png(file=paste("T:/cosmoPaper/figures/",filename,".png",sep=""), width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
    #if(PRINT){ png(file=paste("T:/cosmoPaper/raso/paper/raso_",VAR[iv],".png",sep=""), width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
    par(mfrow=c(2,3),omi=c(0,0.35,0,0.06),cex=CEX)
    for( is in (1:length(station$meta$name))[c(3,4,5,6,2,1)] ){
      if(is%in%3:5){ 
        par(mai=c(0.05,0.05,0.6,0.05))
      }else{
        par(mai=c(0.6,0.05,0.05,0.05))
      }
      yindex = seq(1,39,1)
      H = station$data[[is]]$w$h [yindex]
      plot(H*0+999999,H,xlim = XLIM[[iv]],ylim=c(1000,0), ylab="",xlab="",xaxt="n",yaxt="n",xaxs="i",yaxs="i")
      
      ## a,b,c,d)
      text( par("usr")[1:2]%*%c(.9,.1),par("usr")[3:4]%*%c(.1,.90), labels=c("f)","e)","a)","b)","c)","d)")[is], font=2 )
      #text( par("usr")[1:2]%*%c(.5,.5),par("usr")[3:4]%*%c(.1,.90), labels=station$meta$name[is], font=2 )
      
      ## legend ##
      if(is%in%3:5){
        legend( diff(-par("usr")[1:2])*.01+par("usr")[1],
                diff(par("usr")[3:4])*.5+par("usr")[4],
                legend=NAMES[is-2],
                col="black",lwd=LWD[is-2] ,lty=LTY[is-2],bty='n',horiz = TRUE,cex=1,xpd=TRUE) 
      }
      
      ## y axis (left) ##
      abline(h=1:3*250,col="gray")
      if(is%in%c(3,6)){
        axis(2,at=1:4*250,tick=-.4, labels=FALSE)
        #axis(2,at=1:4*250,line=-0.5,lty=0)
        axis(2,at=c(1,3)*250,line=-0.5,lty=0)
        axis(2,at=c(2,4)*250,line=-0.5,lty=0)
        mtext("Pressure (hPa)",side=2,line=1.5)
      }
      
      ## x axis (top) ##
      abline(v=rev(rev(ABLINE[[iv]][-1])[-1]),col="gray")
      if(is%in%3:5){
        corrAT = seq(XLIM[[iv]][1],XLIM[[iv]][2],length.out=5)[-1]
        corrLAB = seq(0,1,length.out=5)[-1]
        axis(3,at=corrAT,tick=-0.4,labels=FALSE ) 
        axis(3,at=corrAT,line=-0.5,labels=corrLAB,lty=0 ) 
        if(VAR[iv]=="dd"){ MTEXT="circ.Corr" }else{ MTEXT="Corr" }
        mtext(MTEXT,side=3,line=1.5,col=COLcorr[2],font=2)
      }
      
      ## x axis (bottom) ##
      abline(v=0,col="darkgrey",lwd=3)
      if(!is%in%3:5){
        axis(1,at=ABLINE[[iv]][-1],tick=-0.4,labels=FALSE)
        axis(1,at=ABLINE[[iv]][-1],line=-0.5,lty=0)
        mtext("Bias",side=1,adj=0.25,line=1.5,col=COLbias[2],font=2)
        mtext("/",side=1,adj=0.46,line=1.5)
        mtext("RMSE",side=1,adj=0.75,line=1.5,col=COLrmse[2],font=2)
      }
      if(!is%in%3:5){
      #if(is==2){
        if(iv==1){ XXLAB=expression("Temperature (C)") }
        if(iv==2){ XXLAB=expression(paste("Wind speed (m s"^"-1",")")) }
        if(iv==3){ XXLAB=expression("Wind direction ()") }
        mtext(XXLAB,side=1,line=2.85)
      }
      
      for( ip in 1:3 ){
        if(is%in%c(1,5)){ HOURS = 0 }else{ HOURS = 12 }
        if(NAMES[ip]=="C05 winter"){MONTH = 4:9}
        if(NAMES[ip]=="C15 winter"){MONTH = 4:9}
        if(NAMES[ip]=="C15 summer"){MONTH = c(1:3,10:12)}
        TIMES1 = as.numeric(strftime(station$data[[is]]$time,"%m",tz="UTC"))%in%MONTH
        TIMES2 = as.numeric(strftime(station$data[[is]]$time,"%H",tz="UTC"))%in%HOURS
        for(TIMEVAR in list(2002:2016)){
          TIMES3 = as.numeric(strftime(station$data[[is]]$time,"%Y",tz="UTC"))%in%TIMEVAR
          TIMES = TIMES1 & TIMES2 & TIMES3
          A = station$data[[is]] [[RES[ip]]] [[VAR[iv]]] [TIMES,]
          B = station$data[[is]] [["w"]] [[VAR[iv]]] [TIMES,]
          if(VAR[iv]=="dd"){
            delete = station$data[[is]] [["w"]] [["ff"]] [TIMES,] < 0.5 | station$data[[is]] [[RES[ip]]] [["ff"]] [TIMES,] < 0.5
            A[delete] = NA
            B[delete] = NA
          }
          diff = A - B
          if(VAR[iv]=="dd"){
            diff = diff -(diff> 180)*360 +(diff< -180)*360
          }
          ## remove month mean for corr in case of T ###
          if(VAR[iv]=="t"){
            for( yy in TIMEVAR ){
              for( mm in MONTH ){
                index = as.numeric(strftime(station$data[[is]]$time[TIMES],"%m",tz="UTC"))==mm & as.numeric(strftime(station$data[[is]]$time[TIMES],"%Y",tz="UTC"))==yy
                if(length(which(index))==1){
                  A[index,] = 0
                  B[index,] = 0
                }else{
                  A[index,] = A[index,] - rep( apply(A[index,],2,mean,na.rm=TRUE) , each=dim(A[index,])[1] )
                  B[index,] = B[index,] - rep( apply(B[index,],2,mean,na.rm=TRUE) , each=dim(B[index,])[1] )
                }
              }
            }
          }
          
          corr = NULL
          for( ic in 1:length(H) ){
            index = !is.na(diff[,ic])
            if(length(which(index))>1){
              corr[ic] = cor(A[index,ic],B[index,ic])
              if(VAR[iv]=="dd"){
                corr[ic] = cor.circular(circular(A[index,ic],units="degree"),circular(B[index,ic],units="degree"))
              }
            }else{
              corr[ic] = NA
            }
            
          }
          ndata = apply( diff , 2 , function(x) length(which(!is.na(x))))
          print(c(median(ndata) , ndata[ndata < median(ndata)/2]) )
          diff[ , ndata < median(ndata)/2 ] = NA
          corr[ ndata < median(ndata)/2 ] = NA
          bias = apply( diff  , 2 , mean, na.rm="TRUE" )
          rmse = sqrt(apply( diff^2  , 2 , mean, na.rm="TRUE" ) )
          if(ip==3){
            points( bias[yindex] , H , col=COLbias[ip] ,cex=.2, pch=19)
            points( rmse[yindex] , H , col=COLrmse[ip] ,cex=.2, pch=19)
            points( corr*diff(XLIM[[iv]])+min(XLIM[[iv]]) , H , col=COLcorr[ip], cex=.2, pch=19)
          }else{
            lines( bias[yindex] , H , col=COLbias[ip] ,lwd=LWD[1], lty=LTY[ip])
            lines( rmse[yindex] , H , col=COLrmse[ip] ,lwd=LWD[2], lty=LTY[ip])
            lines( corr*diff(XLIM[[iv]])+min(XLIM[[iv]]) , H , col=COLcorr[ip], lwd=LWD[3], lty=LTY[ip])
          }
          
        }#TIMEVAR
      }#ip
    }#is
    
    if(PRINT) dev.off()
  }#var
}

## test 18h ERA
if(0){
  PRINT = 1
  CEX = 0.8
  #RES = c("w","C1518","C05","C15","C15","T15")
  RES = c("C15","C15","C05")
  RES = c("C15","C1518","C1518")
  NAMES = c("C15 winter","18h summer","18h winter")
  COLbias = c("darkorchid1","darkorchid3","darkorchid4")
  COLrmse = c("coral2","coral3","coral4")
  COLcorr = c("seagreen1","seagreen3","seagreen4")
  LWD = c(2,2,2)
  LTY = c(1,2,3)
  VAR = c("t","ff","dd","q","p","g")
  XLIM = list(c(-3,9),c(-3,9),c(-50,150),c(-0.001,0.001),c(-0.1,0.1),c(-20,20))
  ABLINE = list(c(seq(-3,9,3)),seq(-3,9,3),seq(-50,150,50),c(-0.001,0.001),c(-0.1,0.1),c(-20,20))
  for(iv in 1:3){
    WIDTH=5
    HEIGHT=4
    #if(PRINT){ pdf(paste("T:/cosmoPaper/raso/paper/raso_",VAR[iv],".pdf",sep=""),width=WIDTH,height=HEIGHT) }
    if(PRINT){ png(file=paste("T:/cosmoPaper/raso/paper/raso_",VAR[iv],"x.png",sep=""), width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
    par(mfrow=c(2,3),omi=c(0,0.35,0,0.06),cex=CEX)
    for( is in (1:length(station$meta$name))[c(3,4,5,6,2,1)] ){
      if(is%in%3:5){ 
        par(mai=c(0.05,0.05,0.6,0.05))
      }else{
        par(mai=c(0.6,0.05,0.05,0.05))
      }
      yindex = seq(1,39,1)
      H = station$data[[is]]$w$h [yindex]
      plot(H*0+999999,H,xlim = XLIM[[iv]],ylim=c(1000,0), ylab="",xlab="",xaxt="n",yaxt="n",xaxs="i",yaxs="i")
      
      ## a,b,c,d)
      text( par("usr")[1:2]%*%c(.9,.1),par("usr")[3:4]%*%c(.1,.90), labels=c("f)","e)","a)","b)","c)","d)")[is], font=2 )
      #text( par("usr")[1:2]%*%c(.5,.5),par("usr")[3:4]%*%c(.1,.90), labels=station$meta$name[is], font=2 )
      
      ## legend ##
      if(is%in%3:5){
        legend( diff(-par("usr")[1:2])*.01+par("usr")[1],
                diff(par("usr")[3:4])*.5+par("usr")[4],
                legend=NAMES[is-2],
                col="black",lwd=LWD[is-2] ,lty=LTY[is-2],bty='n',horiz = TRUE,cex=1,xpd=TRUE) 
      }
      
      ## y axis (left) ##
      abline(h=1:3*250,col="gray")
      if(is%in%c(3,6)){
        axis(2,at=1:4*250,tick=-.4, labels=FALSE)
        #axis(2,at=1:4*250,line=-0.5,lty=0)
        axis(2,at=c(1,3)*250,line=-0.5,lty=0)
        axis(2,at=c(2,4)*250,line=-0.5,lty=0)
        mtext("Pressure (hPa)",side=2,line=1.5)
      }
      
      ## x axis (top) ##
      abline(v=rev(rev(ABLINE[[iv]][-1])[-1]),col="gray")
      if(is%in%3:5){
        corrAT = seq(XLIM[[iv]][1],XLIM[[iv]][2],length.out=5)[-1]
        corrLAB = seq(0,1,length.out=5)[-1]
        axis(3,at=corrAT,tick=-0.4,labels=FALSE ) 
        axis(3,at=corrAT,line=-0.5,labels=corrLAB,lty=0 ) 
        mtext("Corr",side=3,line=1.5,col=COLcorr[2],font=2)
      }
      
      ## x axis (bottom) ##
      abline(v=0,col="darkgrey",lwd=3)
      if(!is%in%3:5){
        axis(1,at=ABLINE[[iv]][-1],tick=-0.4,labels=FALSE)
        axis(1,at=ABLINE[[iv]][-1],line=-0.5,lty=0)
        mtext("Bias",side=1,adj=0.25,line=1.5,col=COLbias[2],font=2)
        mtext("/",side=1,adj=0.46,line=1.5)
        mtext("RMSE",side=1,adj=0.75,line=1.5,col=COLrmse[2],font=2)
      }
      if(!is%in%3:5){
        #if(is==2){
        if(iv==1){ XXLAB=expression("Temperature (K)") }
        if(iv==2){ XXLAB=expression(paste("Wind speed (m s"^"-1",")")) }
        if(iv==3){ XXLAB=expression("Wind direction ()") }
        mtext(XXLAB,side=1,line=2.85)
      }
      
      for( ip in 1:3 ){
        if(is%in%c(1,5)){ HOURS = 0 }else{ HOURS = 12 }
        if(NAMES[ip]=="18h winter"){MONTH = 4:9}
        if(NAMES[ip]=="C15 winter"){MONTH = 4:9}
        if(NAMES[ip]=="18h summer"){MONTH = c(1:3,10:12)}
        TIMES1 = as.numeric(strftime(station$data[[is]]$time,"%m",tz="UTC"))%in%MONTH
        TIMES2 = as.numeric(strftime(station$data[[is]]$time,"%H",tz="UTC"))%in%HOURS
        for(TIMEVAR in list(2002:2015)){
          TIMES3 = as.numeric(strftime(station$data[[is]]$time,"%Y",tz="UTC"))%in%TIMEVAR
          TIMES = TIMES1 & TIMES2 & TIMES3
          A = station$data[[is]] [[RES[ip]]] [[VAR[iv]]] [TIMES,]
          B = station$data[[is]] [["w"]] [[VAR[iv]]] [TIMES,]
          if(VAR[iv]=="dd"){
            delete = station$data[[is]] [["w"]] [["ff"]] [TIMES,] < 0.5 | station$data[[is]] [[RES[ip]]] [["ff"]] [TIMES,] < 0.5
            A[delete] = NA
            B[delete] = NA
          }
          diff = A - B
          ## remove month mean for corr ###
          for( yy in TIMEVAR ){
            for( mm in MONTH ){
              index = as.numeric(strftime(station$data[[is]]$time[TIMES],"%m",tz="UTC"))==mm & as.numeric(strftime(station$data[[is]]$time[TIMES],"%Y",tz="UTC"))==yy
              if(length(which(index))==1){
                A[index,] = 0
                B[index,] = 0
              }else{
                A[index,] = A[index,] - rep( apply(A[index,],2,mean,na.rm=TRUE) , each=dim(A[index,])[1] )
                B[index,] = B[index,] - rep( apply(B[index,],2,mean,na.rm=TRUE) , each=dim(B[index,])[1] )
              }
            }
          }
          corr = NULL
          for( ic in 1:length(H) ){
            index = !is.na(diff[,ic])
            corr[ic] = cor(A[index,ic],B[index,ic])
          }
          ndata = apply( diff , 2 , function(x) length(which(!is.na(x))))
          print(c(median(ndata) , ndata[ndata < median(ndata)/2]) )
          diff[ , ndata < median(ndata)/2 ] = NA
          corr[ ndata < median(ndata)/2 ] = NA
          if(VAR[iv]=="dd"){
            diff = diff -(diff> 180)*360 +(diff< -180)*360
          }
          bias = apply( diff  , 2 , mean, na.rm="TRUE" )
          rmse = sqrt(apply( diff^2  , 2 , mean, na.rm="TRUE" ) )
          if(ip==3){
            points( bias[yindex] , H , col=COLbias[ip] ,cex=.2, pch=19)
            points( rmse[yindex] , H , col=COLrmse[ip] ,cex=.2, pch=19)
            points( corr*diff(XLIM[[iv]])+min(XLIM[[iv]]) , H , col=COLcorr[ip], cex=.2, pch=19)
          }else{
            lines( bias[yindex] , H , col=COLbias[ip] ,lwd=LWD[1], lty=LTY[ip])
            lines( rmse[yindex] , H , col=COLrmse[ip] ,lwd=LWD[2], lty=LTY[ip])
            lines( corr*diff(XLIM[[iv]])+min(XLIM[[iv]]) , H , col=COLcorr[ip], lwd=LWD[3], lty=LTY[ip])
          }
          
        }#TIMEVAR
      }#ip
    }#is
    
    if(PRINT) dev.off()
  }#var
}

## plot mean profile P
## dd mean does not make super sense?
if(0){
  PRINT = 1
  
  RES = c("w","w","C15","C15","C1518","C1518")
  NAMES = c("Raso winter","Raso summer","C15 winter","C15 summer","18h winter","18h summer")
  CEX = 0.8
  COL = rep(c("black","dodgerblue","orange"),each=2)
  LWD = 2
  LTYwinter = 1
  LTYsummer = 2
  VAR = c("t","ff","dd","q","p","g","theta")
  XLIM = list(c(-85,0),c(0,40),c(0,360),c(-0.001,0.001),c(0.1,0.1),c(-20,20),c(-60,-35))
  XTICKS = list(c(seq(-80,0,10)),seq(0,40,5),seq(0,360,45),c(-0.001,0.001),c(-0.1,0.1),c(-20,20),c(seq(-80,5,5)))
  XAT = list(c(seq(-80,0,20)),seq(0,40,10),seq(0,360,90),c(-0.001,0.001),c(-0.1,0.1),c(-20,20),c(seq(-80,5,5)))
  
  for(iv in 3){
    WIDTH=5
    HEIGHT=4
    #if(PRINT){ pdf(paste("T:/cosmoPaper/raso/paper/raso_profile_",VAR[iv],".pdf",sep=""),width=WIDTH,height=HEIGHT) }
    if(PRINT){ png(file=paste("T:/cosmoPaper/raso/paper/raso_profile_",VAR[iv],"x.png",sep=""), width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
    par(mfrow=c(2,3),omi=c(0,0.35,0,0.07),cex=CEX)
    for( is in (1:length(station$meta$name))[c(3,4,5,6,2,1)] ){
      H = station$data[[is]]$w$h

      if(is%in%3:5){ 
        par(mai=c(0.15,0.05,0.45,0.05))
      }else{
        par(mai=c(0.45,0.05,0.15,0.05))
      }
      plot(H*0+999999,H,xlim = XLIM[[iv]],ylim=c(1000,0), ylab="",xlab="",xaxt="n",yaxt="n",xaxs="i",yaxs="i")
      
      ## a,b,c,d)
      text( par("usr")[1:2]%*%c(.9,.1),par("usr")[3:4]%*%c(.1,.90), labels=c("f)","e)","a)","b)","c)","d)")[is], font=2 )
      #text( par("usr")[1:2]%*%c(.5,.5),par("usr")[3:4]%*%c(.1,.90), labels=station$meta$name[is], font=2 )
      
      ## legend ##
      if(is%in%3:5){
        legend( diff(-par("usr")[1:2])*-.0+par("usr")[1],
                diff(par("usr")[3:4])*.35+par("usr")[4],
                legend=NAMES[(is-2)*2+-1:0],
                col=COL[(is-2)*2],lwd=LWD ,lty=c(LTYwinter,LTYsummer),bty='n',horiz = FALSE,cex=1,xpd=TRUE)
      }
      
      ## y axis (left) ##
      abline(h=1:3*250,col="gray")
      if(is%in%c(3,6)){
        axis(2,at=1:4*250,tick=-.4, labels=FALSE)
        #axis(2,at=1:4*250,line=-0.5,lty=0)
        axis(2,at=c(1,3)*250,line=-0.5,lty=0)
        axis(2,at=c(2,4)*250,line=-0.5,lty=0)
        mtext("Pressure (hPa)",side=2,line=1.5)
      }
      
      
      ## x axis (bottom) ##
      abline(v=rev(rev(XTICKS[[iv]][-1])[-1]),col="gray")
      axis(1,at=XTICKS[[iv]][-1],tck=-0.03,labels=FALSE)
      axis(1,at=XAT[[iv]][-1],tck=-0.06,labels=FALSE)
      axis(1,at=XAT[[iv]][-1],line=-0.5,lty=0)
      if(!is%in%3:5){
        if(iv==1){ XXLAB=expression("Temperature (C)") }
        if(iv==2){ XXLAB=expression(paste("Wind speed (m s"^"-1",")")) }
        if(iv==3){ XXLAB=expression("Wind Direction (deg)") }
        mtext(XXLAB,side=1,line=1.9)
      }
      
      for( ip in 1:6 ){
        if(is%in%c(1,5)){ HOURS = 0 }else{ HOURS = 12 }
        if(NAMES[ip]=="Raso winter"){MONTH = 4:9}
        if(NAMES[ip]=="Raso summer"){MONTH = c(1:3,10:12)}
        if(NAMES[ip]=="C15 winter"){MONTH = 4:9}
        if(NAMES[ip]=="C15 summer"){MONTH = c(1:3,10:12)}
        if(NAMES[ip]=="T15 winter"){MONTH = 4:9}
        if(NAMES[ip]=="T15 summer"){MONTH = c(1:3,10:12)}
        TIMES1 = as.numeric(strftime(station$data[[is]]$time,"%m",tz="UTC"))%in%MONTH
        TIMES2 = as.numeric(strftime(station$data[[is]]$time,"%H",tz="UTC"))%in%HOURS
        for(TIMEVAR in list(2002:2016)){
          TIMES3 = as.numeric(strftime(station$data[[is]]$time,"%Y",tz="UTC"))%in%TIMEVAR
          TIMES = TIMES1 & TIMES2 & TIMES3
          
          if(VAR[iv]=="theta"){
            p_0 = 1000#00
            R_L = 287
            c_p = 1003
            A = station$data[[is]] [[RES[ip]]] [["t"]] [TIMES,] * (p_0/station$data[[is]] [[RES[ip]]] [["p"]] [TIMES,])^(R_L/c_p)
          }else{          
            A = station$data[[is]] [[RES[ip]]] [[VAR[iv]]] [TIMES,]
          }
          
          ndata = apply( A , 2 , function(x) length(which(!is.na(x))))
          print(c(median(ndata) , ndata[ndata < median(ndata)/2]) )
          A[ , ndata < median(ndata)/2 ] = NA
          temp = apply( A  , 2 , mean, na.rm="TRUE" )
          if( all(MONTH==4:9) ){ LTY=LTYwinter }
          if( all(MONTH==c(1:3,10:12)) ){ LTY=LTYsummer }
          lines( temp , H , col=COL[ip] ,lwd=LWD, lty=LTY)
          
          #if(ip==2){text(weighted.mean(par("usr")[1:2],c(.45,.55)),weighted.mean(par("usr")[3:4],c(.97,.02)),paste(NAMES[ip]," n=",round(median(ndata)/100)*100,sep=""),col=COL[ip-1],font=2 )}
          #if(ip==3){text(weighted.mean(par("usr")[1:2],c(.15,.85)),weighted.mean(par("usr")[3:4],c(.97,.02)),paste(NAMES[ip]," n=",round(median(ndata)/100)*100,sep=""),col=COL[ip-1],font=2 )}
          if(is==1){
            xStart = min(XLIM[[iv]]) + diff(XLIM[[iv]]*0.00)
            xLength = xStart + c(0.05,0.25)*diff(XLIM[[iv]])
          }
          
        }
      }
    }
    if(PRINT) dev.off()
  }
}


# load data
 load("T:/cosmoPaper/raso/rdata/stationsHOG.rData")
if(1){
  PRINT = 1
  
  RES = c("w","w","C15","C15","T15","T15")
  NAMES = c("Raso winter","Raso summer","C15 winter","C15 summer","T15 winter","T15 summer")
  CEX = 0.8
  COL = rep(c("black","dodgerblue","forestgreen"),each=2)
  LWD = 2
  LTYwinter = 1
  LTYsummer = 2
  VAR = c("t","ff","dd","q","p","g","theta")
  XLIM1 = list(c(-30,0),c(-2,8),c(-50,100),c(-0.001,0.001),c(0.1,0.1),c(-20,20),c(-60,-35))
  XLIM2 = list(c(-60,-30),c(-2,8),c(-50,100),c(-0.001,0.001),c(0.1,0.1),c(-20,20),c(-60,-35))
  XTICKS1 = list(c(seq(-30,0,5)),seq(-2,8,1),seq(-50,100,10),c(-0.001,0.001),c(-0.1,0.1),c(-20,20),c(seq(-80,5,5)))
  XTICKS2 = list(c(seq(-60,0,5)),seq(-2,8,1),seq(-50,100,10),c(-0.001,0.001),c(-0.1,0.1),c(-20,20),c(seq(-80,5,5)))
  XAT1 = list(c(seq(-30,0,10)),seq(-2,8,1),seq(-50,100,10),c(-0.001,0.001),c(-0.1,0.1),c(-20,20),c(seq(-80,5,5)))
  XAT2 = list(c(seq(-60,0,10)),seq(-2,8,1),seq(-50,100,10),c(-0.001,0.001),c(-0.1,0.1),c(-20,20),c(seq(-80,5,5)))
  
  for(iv in 1){
    WIDTH=5
    HEIGHT=4
    if(PRINT){ pdf(paste("T:/cosmoPaper/figures/fig11.pdf",sep=""),width=WIDTH,height=HEIGHT) }
    #if(PRINT){ png(file=paste("T:/cosmoPaper/raso/paper/raso_meter_",VAR[iv],".png",sep=""), width=WIDTH*300 ,height=HEIGHT*300,res = 300) }
    par(mfrow=c(2,3),omi=c(0,0.35,0,0.07),cex=CEX)
    for( is in (1:length(station$meta$name))[c(3,4,5,6,2,1)] ){
      H = station$data[[is]]$w$h
      #plot(H*0+999999,H,xlim = XLIM[[iv]]+30*(is>1),ylim=c(0,1700) ,cex.axis=CEX, ylab="",xlab="")
      
      if(is%in%3:5){ 
        par(mai=c(0.15,0.05,0.45,0.05))
      }else{
        par(mai=c(0.45,0.05,0.15,0.05))
      }
      if(is==1){ 
        XLIM = XLIM2
        XAT = XAT2
        XTICKS = XTICKS2
      }else{ 
        XLIM = XLIM1 
        XAT = XAT1
        XTICKS = XTICKS1
      }
      plot(H*0+999999,H,xlim = XLIM[[iv]],ylim=c(0,1200), ylab="",xlab="",xaxt="n",yaxt="n",xaxs="i",yaxs="i")
      
      ## a,b,c,d)
      text( par("usr")[1:2]%*%c(.9,.1),par("usr")[3:4]%*%c(.1,.90), labels=c("f)","e)","a)","b)","c)","d)")[is], font=2 )
      #text( par("usr")[1:2]%*%c(.5,.5),par("usr")[3:4]%*%c(.1,.90), labels=station$meta$name[is], font=2 )
      
      ## legend ##
      if(is%in%3:5){
        legend( diff(-par("usr")[1:2])*-.0+par("usr")[1],
                diff(par("usr")[3:4])*.35+par("usr")[4],
                legend=NAMES[(is-2)*2+-1:0],
                col=COL[(is-2)*2],lwd=LWD ,lty=c(LTYwinter,LTYsummer),bty='n',horiz = FALSE,cex=1,xpd=TRUE)
      }
      
      ## y axis (left) ##
      abline(h=1:5*200,col="gray")
      if(is%in%c(3,6)){
        axis(2,at=0:5*200,tck=-0.03, labels=FALSE)
        axis(2,at=c(0,2,4)*200,tck=-0.06, labels=FALSE)
        axis(2,at=c(0,2,4)*200,line=-0.5,lty=0)
        #axis(2,at=c(0,2)*200,line=-0.5,lty=0)
        mtext("Height AGL(m)",side=2,line=1.5)
      }
      
      
      ## x axis (bottom) ##
      abline(v=rev(rev(XTICKS[[iv]][-1])[-1]),col="gray")
      axis(1,at=XTICKS[[iv]][-1],tck=-0.03,labels=FALSE)
      axis(1,at=XAT[[iv]][-1],tck=-0.06,labels=FALSE)
      axis(1,at=XAT[[iv]][-1],line=-0.5,lty=0)
      if(!is%in%3:5){
        if(iv==1){ XXLAB=expression("Temperature (C)") }
        if(iv==2){ XXLAB=expression(paste("Wind speed (m s"^"-1",")")) }
        if(iv==3){ XXLAB=expression("Wind Direction (deg)") }
        mtext(XXLAB,side=1,line=1.9)
      }
      
      for( ip in 1:6 ){
        if(is%in%c(1,5)){ HOURS = 0 }else{ HOURS = 12 }
        if(NAMES[ip]=="Raso winter"){MONTH = 4:9}
        if(NAMES[ip]=="Raso summer"){MONTH = c(1:3,10:12)}
        if(NAMES[ip]=="C15 winter"){MONTH = 4:9}
        if(NAMES[ip]=="C15 summer"){MONTH = c(1:3,10:12)}
        if(NAMES[ip]=="T15 winter"){MONTH = 4:9}
        if(NAMES[ip]=="T15 summer"){MONTH = c(1:3,10:12)}
        TIMES1 = as.numeric(strftime(station$data[[is]]$time,"%m",tz="UTC"))%in%MONTH
        TIMES2 = as.numeric(strftime(station$data[[is]]$time,"%H",tz="UTC"))%in%HOURS
        for(TIMEVAR in list(2002:2016)){
          TIMES3 = as.numeric(strftime(station$data[[is]]$time,"%Y",tz="UTC"))%in%TIMEVAR
          TIMES = TIMES1 & TIMES2 & TIMES3
          
          if(VAR[iv]=="theta"){
            p_0 = 1000#00
            R_L = 287
            c_p = 1003
            A = station$data[[is]] [[RES[ip]]] [["t"]] [TIMES,] * (p_0/station$data[[is]] [[RES[ip]]] [["p"]] [TIMES,])^(R_L/c_p)
          }else{          
            A = station$data[[is]] [[RES[ip]]] [[VAR[iv]]] [TIMES,]
          }
          
          ndata = apply( A , 2 , function(x) length(which(!is.na(x))))
          print(c(median(ndata) , ndata[ndata < median(ndata)/2]) )
          A[ , ndata < median(ndata)/2 ] = NA
          temp = apply( A  , 2 , mean, na.rm="TRUE" )
          if( all(MONTH==4:9) ){ LTY=LTYwinter }
          if( all(MONTH==c(1:3,10:12)) ){ LTY=LTYsummer }
          lines( temp , H , col=COL[ip] ,lwd=LWD, lty=LTY)
          
          #if(ip==2){text(weighted.mean(par("usr")[1:2],c(.45,.55)),weighted.mean(par("usr")[3:4],c(.97,.02)),paste(NAMES[ip]," n=",round(median(ndata)/100)*100,sep=""),col=COL[ip-1],font=2 )}
          #if(ip==3){text(weighted.mean(par("usr")[1:2],c(.15,.85)),weighted.mean(par("usr")[3:4],c(.97,.02)),paste(NAMES[ip]," n=",round(median(ndata)/100)*100,sep=""),col=COL[ip-1],font=2 )}
          if(is==1){
            xStart = min(XLIM[[iv]]) + diff(XLIM[[iv]]*0.00)
            xLength = xStart + c(0.05,0.25)*diff(XLIM[[iv]])
          }
          
        }
      }
    }
    if(PRINT) dev.off()
  }
}

