library(data.table)
library(doParallel) 
nproc = 60
cl <- makeCluster(nproc, outfile="")
registerDoParallel(cl)

inpath="/home/sida6serv02/zentek/cosmoPaper/raso/wyoming/org"
outpath="/home/sida6serv02/zentek/cosmoPaper/raso/wyoming/rdata"

STATIONNAME = unique(sub("_.......txt","",dir(inpath)))

stations = list()
for( station in 1:length(STATIONNAME) ){
  files = paste(inpath,dir(inpath)[grep(STATIONNAME[station],dir(inpath))],sep="/")

  counterData = 1
  print(STATIONNAME[station])
    
  DATA = foreach( file = files, .combine=function(x,y){return(list(data=c(x$data,y$data),meta=rbind(x$meta,y$meta)))}, .packages="data.table") %dopar% {
    cat(paste(round(which(file==files)/length(files)*100),"% ",collapse=""))
    startlines = as.numeric(system(paste("grep -n H2",file,"| sed 's/:.*//'"),intern=TRUE))
    if( length(startlines)==0 ) return(list(data=NULL,meta=NULL))
    endlines = as.numeric(system(paste("grep -n H3",file,"| sed 's/:.*//'"),intern=TRUE))
    data = list()
    meta = matrix(NA,ncol=5,nrow=as.numeric(system(paste("grep H3",paste(file,collapse=" "),"| wc | awk '{print $1}'"),intern=TRUE)))
    for( raso in 1:length(startlines) ){
      data[[raso]] = read.fwf(file, width=rep(7,11),
                                skip=5+startlines[raso],
                                nrow=-6+endlines[raso]-startlines[raso],
                                stringsAsFactors=FALSE,
                                col.names=c("p","h","t","d","rh","mr","dd","ff","pt","et","vt"))
      data[[raso]]$ff = 0.514444 * data[[raso]]$ff
      meta[raso,] = read.table(file, skip=endlines[raso], nrow=5, sep=":", stringsAsFactors=FALSE)[,2]
    }
    return(list(data=data,meta=meta))
  }
  meta = DATA$meta
  data = DATA$data
  meta = data.table(stationNumber=as.numeric(meta[,1]), time=as.POSIXct(strptime(meta[,2],"%y%m%d/%H%M",tz="GMT")), lat=as.numeric(meta[,3]), lon=as.numeric(meta[,4]),
                    elevation=as.numeric(meta[,5]))
  
  stations[[station]] = list(name=STATIONNAME[station],meta=meta,data=data)
}

stopCluster(cl)

save(stations,file=paste(outpath,"stations.rData",sep="/"))
