library(ncdf4)
library(data.table)
source("T:/cosmoPaper/map/geo2rot.R")

# clean up noaa
load("T:/cosmoPaper/surface/noaa/rdata/stations.rData")
station$meta[, c("STATION_ID","BEGIN_DATE","END_DATE","STATE","COUNTRY"):=NULL]  # remove two columns
names(station$meta)[names(station$meta) %in% c("STATION","LATITUDE","LONGITUDE","ELEVATION")] = c("name","lat","lon","height")
station$meta$source="noaa"

# merge bas into it
for( file in c("bas_aws.rData","bas_surface.rData") ){
  load(paste("T:/cosmoPaper/surface/bas/rdata/",file,sep=""))
  for( is in seq(stations) ){
    if( length(stations[[is]]$data$time)==0 ){next}
    station$data[[length(station$data)+1]] = stations[[is]]$data
    station$meta = rbind(station$meta, list(stations[[is]]$name,stations[[is]]$lat,stations[[is]]$lon,stations[[is]]$height,NA,NA,sub(".rData","",file)))
  }
}
station$meta$height = as.numeric(station$meta$height)
out = geo2rot(0,12.5,station$meta$lon,station$meta$lat,180)
station$meta$rlon = out$x
station$meta$rlat = out$y

# remove outside domain
nc = nc_open(paste("S:/COSMO_","C15","_out01/20040404/out01/lffd2004040318c.nc",sep=""))
rlon = ncvar_get(nc,"rlon")
rlat = ncvar_get(nc,"rlat")
nc_close(nc)
keep = (min(rlon) < station$meta$rlon) & (station$meta$rlon < max(rlon)) & (min(rlat) < station$meta$rlat) & (station$meta$lat < max(rlat))
station$meta = station$meta[keep]
station$data = station$data[keep]

# remove data before 2002
keep = NULL
for( is in seq(station$meta$height) ){
  station$data[[is]] = station$data[[is]][station$data[[is]]$time >= as.POSIXct("2002/01/01 00:00:00",tz="UTC")]
  if( length(station$data[[is]]$time)>0 ){ keep = c(keep,is) }
}
station$meta = station$meta[keep]
station$data = station$data[keep]

# sort by lat and lon
station$data = station$data[sort(station$meta$rlon,index.return=TRUE)$ix]
station$meta = station$meta[sort(station$meta$rlon,index.return=TRUE)$ix]
station$data = station$data[sort(station$meta$rlat,index.return=TRUE)$ix]
station$meta = station$meta[sort(station$meta$rlat,index.return=TRUE)$ix]

save(station,file=paste("T:/cosmoPaper/surface/rdata/stationOBS.rData"))

