#REMOTE=TRUE
REMOTE=FALSE

library(ncdf4)
library(data.table)
library(doParallel)
if(REMOTE){
  source("T:/cosmoPaper/map/geo2rot.R")
  source("T:/cosmoPaper/map/rotateWind.R")
  path = "T:/cosmoPaper/surface"
  cclmPath = list( C15="S:/COSMO_" , T15="R:/COSMO_" , C05="R:/COSMO_" )
}else{
  source("/home/cosmoPaper/map/geo2rot.R")
  source("/home/cosmoPaper/map/rotateWind.R")
  path="/home/cosmoPaper/surface"
  cclmPath = list( C15="/home/sida6serv02/zentek/cosmo/COSMO_" , T15="/home/sida6serv02/zentek/cosmo/COSMO_" , C05="/home/sida6serv02/zentek/cosmo/COSMO_" )
}

load(paste(path,"/rdata/station.rData",sep=""))

### with 60 procs on sida one run 2002-2016 takes about 19 hours... C05 was faster
nproc = 60
cl <- makeCluster(nproc, outfile="")
registerDoParallel(cl)

# META DATA
meta = station$meta

#### MAIN PROGRAM ###
for( RES in c("C05","T15","C15")[3] ){
  
  print(paste("===== ",RES," ====="))
  file = paste(cclmPath[[RES]],RES,"_out01/20040404/out01/lffd2004040318c.nc",sep="")
  
  nc = nc_open(file)
  lon = ncvar_get(nc,"lon")
  lat = ncvar_get(nc,"lat")
  nc_close(nc)
  
  for( is in seq(meta$name) ){
    # find cosmo grid points nearest to station
    dist = sqrt(((lon-meta$lon[is])*cos(meta$lat[is]/180*pi))^2+(lat-meta$lat[is])^2)
    dist = which(dist == min(dist) , arr.ind = TRUE)
    meta$ix[is] = dist[1]
    meta$iy[is] = dist[2]
  }
  station$meta=meta
  
  #, .combine=function(x,y){for(i in seq(x)){x[[i]]=rbind(x[[i]],y[[i]])}; return(x)}
  data = foreach( is=seq(meta$name) , .packages=c("data.table","ncdf4") ) %dopar% {
    
    # show percentage
    cat( paste(round(is/length(meta$name)*100),"% ",sep="") )
    
    # loop over the time
    TIME = seq( as.POSIXct("2002/01/01 00:00",tz="UTC") , as.POSIXct("2016/12/31 23:00",tz="UTC") , "hour" )#[1:72]
    # subselect time for C05
    if( RES=="C05" ) {
      TIME = TIME[ as.numeric(strftime(TIME,"%m",tz="UTC"))%in%4:9 ]
    }

    VAR = c("U_10M","V_10M","T_2M","PMSL","PS")
    cdata = matrix(numeric(),nrow = length(TIME), ncol = length(VAR))
    # subselect station for C05
    if( RES=="C05" ) {
      if( meta$ix[is]%in%c(1,500) | meta$iy[is]%in%c(1,500) ){
        cdata = setNames(data.table(cdata), VAR)
        cdata = cbind(TIME,cdata)
        return(cdata)
      }
    }
	
    for( it in 1:length(TIME) ){
      file = paste(cclmPath[[RES]],RES,"/",strftime(TIME[it],"%Y%m%d",tz="UTC"),"/out02/lffd",strftime(TIME[it],"%Y%m%d%H",tz="UTC"),".nc",sep="")
      nc = nc_open(file)
      for( ivar in seq(VAR) ){
        cdata[it,ivar] = ncvar_get(nc,VAR[ivar],start=c(meta$ix[is],meta$iy[is],1),c(1,1,1))
      }# ivar
      nc_close(nc)
    }# it
    
    cdata = setNames(data.table(cdata), VAR)
    cdata = cbind(TIME,cdata)
    
    return(cdata)
  }# dopar
  cclm = list(meta=meta,data=data)
  dir.create(paste(path,"rdata",sep="/"), showWarnings = FALSE, recursive=TRUE)
  save(cclm,file=paste(path,"/rdata/station",RES,".rData",sep=""))
}# RES



stopCluster(cl)
