library(ncdf4)
library(data.table)
library(circular)

# INIT
if(0){
  load("T:/cosmoPaper/surface/rdata/stationFinal.rData")
  
  RES="C15"
  file = paste("S:/COSMO_",RES,"_out01/20040404/out01/lffd2004040318c.nc",sep="")
  nc = nc_open(file)
  land = ncvar_get(nc,"FR_LAND")
  hsurf = ncvar_get(nc,"HSURF")
  rlon = ncvar_get(nc,"rlon")
  rlat = ncvar_get(nc,"rlat")
  nc_close(nc)
  
  RES="C05"
  file = paste("S:/COSMO_",RES,"_out01/20040404/out01/lffd2004040318c.nc",sep="")
  nc = nc_open(file)
  land5 = ncvar_get(nc,"FR_LAND")
  hsurf5 = ncvar_get(nc,"HSURF")
  nc_close(nc)
  
}

### Chosen
# south # east coast # south Peninsula # middle peninsula # north peninsula
chosen = c(5,12 , 47,65,107 , 33,45,30 , 41,43,50,53 , 76,78,81,83,87,89)
color  = 1+c(1,1  ,  2, 2,  2 ,  3, 3, 3 ,  4, 4, 4, 4 ,  5, 5, 5, 5, 5, 5)
xoff = c(rep(0,8)                      ,  -.5,.5,0,0 ,  0,-1,-1,-1,0,-1)
yoff = c(rep(0.5,12)                                 ,  0.5,-2,-1,0,1,+1)
if(1){
  par(mfrow=c(1,1),omi=c(.1,.1,.1,.1),mai=rep(0,4))

  ### map ###
  image(rlon,rlat,land,col=c("cornflowerblue","gray"),asp=1,xaxt="n",yaxt="n")
  points(station$meta$rlon,station$meta$rlat,pch=20)
  #points(station$meta$rlon[chosenAll],station$meta$rlat[chosenAll],pch=19,col="black")
  text(station$meta$rlon[chosen]+xoff,station$meta$rlat[chosen]+yoff,label=chosen,cex=1,col=color,font=2)
  
}

### make statistics
if(1){
  STATS = list()
  for( RES in c("C05","C15","T15")[] ){
    print(RES)
    STATS[[RES]] = list()
    STATS$meta = station$meta[chosen]
    for( is in seq(chosen)[] ){
      STATS[[RES]][[is]] = list()
      
      x = station$data[[chosen[is]]]
      m = station$meta[chosen[is]]
      
      ### vars ###
      VAR = c("t","ff","dd")
      for(var in VAR[] ){
        STATS[[RES]][[is]][[var]] = list()
        xx = x[ !is.na(x[[var]]) ]
        ##### manual correction #####
        if(var=="dd" & m$name=="Larsen"){
          xx = xx[ xx$time>as.POSIXct("2006/01/01",tz="UTC") ]
        }
        if(RES=="C15"&var=="ff"){removed=round(length(which(xx$ff==0))/length(xx$ff)*100); print(c(is,removed))}
        if(var%in%c("ff","dd")){ xx = xx[ xx$ff>0 ] }
        double = which(duplicated(xx$time))
        if(length(double)>0){ xx = xx[ -which(duplicated(xx$time)) ] }
        cc = station[[RES]][[chosen[is]]][ station[[RES]][[chosen[is]]]$time %in% xx$time ]
        xx = xx[ xx$time %in% cc$time ]
        diff = cc[[var]]-xx[[var]]
        if(length(diff)==0) next
        if(var=="dd"){
          diff = diff + 360*(diff< -180) - 360*(diff> 180)
          keep = cc$ff>.5 & xx$ff>.5
          keep = which( keep )
          diff = diff[ keep ]
          xx = xx[keep]
          cc = cc[keep]
        }
        for( ws in c("w","s") ){
          if( (RES=="C05"&ws=="s") | (RES=="C05"&is%in%1:2) ){next}
          STATS[[RES]][[is]][[var]][[ws]] = list()
          if(ws=="w"){ keep = as.numeric(strftime(xx$time,"%m",tz="UTC")) %in% 4:9 }
          if(ws=="s"){ keep = !(as.numeric(strftime(xx$time,"%m",tz="UTC")) %in% 4:9) }
          
          for( dm in c("d","m")){
            STATS[[RES]][[is]][[var]][[ws]][[dm]] = list()
            
            if(dm=="d"){ mergeOver = "%Y%m%d" }
            if(dm=="m"){ mergeOver = "%Y%m" }
            
            zz = data.table(
              date = strftime(xx$time[keep],mergeOver,tz="UTC"),
              xx = xx[[var]][keep],
              cc = cc[[var]][keep],
              Cxx = circular(xx[[var]][keep],units="degree"),
              Ccc = circular(cc[[var]][keep],units="degree"),
              diff = diff[keep]
            )
            
            MEAN = zz[,mean(diff), by = date]$V1
            RMSE = zz[,sqrt(mean(diff^2)), by = date]$V1
            
            ### remove single data points or cases with sd==0 befor computing CORR
            SDXX = zz[,sd(xx), by = date]$V1
            SDCC = zz[,sd(cc), by = date]$V1
            DATE = zz[,sd(cc), by = date]$date
            BAD = is.na(SDXX) | is.na(SDCC) | SDXX==0 | SDCC==0
            
            zz = zz[ zz$date %in% DATE[!BAD] ]
            
            CORR = zz[,cor(xx,cc), by = date]$V1
            if(var=="dd"){
              CORR = zz[,cor.circular(Cxx,Ccc), by = date]$V1
            }
            CORR = CORR[!is.na(CORR)]
            
            #CCOR = zz[,cor.circular(Cxx,Ccc), by = date]$V1
            #TEST = zz[,sum(xx*0+1), by = date]$V1
            #DATE = zz[,sum(xx*0+1), by = date]$date
            
            STATS[[RES]][[is]][[var]][[ws]][[dm]][["b"]] = ( MEAN )
            STATS[[RES]][[is]][[var]][[ws]][[dm]][["r"]] = ( RMSE )
            STATS[[RES]][[is]][[var]][[ws]][[dm]][["c"]] = ( CORR )
            
          }#dm
          
        }#ws
            
      }#var
    }#is
    meta = station$meta[chosen]
    meta$hc = round(hsurf[ cbind(station$meta$ix[chosen] , station$meta$iy[chosen]) ])
    meta$fr = ( land[ cbind(station$meta$ix[chosen] , station$meta$iy[chosen]) ] )
    if(RES=="C05"){
      meta$hc = round(hsurf5[ cbind(station$meta$ix5[chosen] , station$meta$iy5[chosen]) ])
      meta$fr = ( land5[ cbind(station$meta$ix5[chosen] , station$meta$iy5[chosen]) ] )
    }
    meta$src = station$meta$source[chosen]
    
    STATS[[RES]]$meta = meta
    
  }#RES
  save(STATS,file="T:/cosmoPaper/surface/rdata/statistic.rdata")
}

### old ####
### make plotable data
# if(0){
#   STATS = list()
#   for( RES in c("C05","C15","T15")[] ){
#     a = numeric()
#     stats = data.frame(id=a,name=character(),
#                        t_sb=a,t_wb=a,t_sr=a,t_wr=a,
#                        ff_sb=a,ff_wb=a,ff_sr=a,ff_wr=a,
#                        dd_sb=a,dd_wb=a,dd_sr=a,dd_wr=a,
#                        stringsAsFactors=FALSE)
#     for( is in seq(chosen) ){
#       x = station$data[[chosen[is]]]
#       m = station$meta[chosen[is]]
#       
#       colcount = 0
#       ### vars ###
#       stats[is,colcount<-colcount+1] = chosen[is]
#       stats[is,colcount<-colcount+1] = station$meta$name[chosen[is]]
#       VAR = c("t","ff","dd")
#       for(var in VAR ){
#         xx = x[ !is.na(x[[var]]) ]
#         double = which(duplicated(xx$time))
#         if(length(double)>0){ xx = xx[ -which(duplicated(xx$time)) ] }
#         cc = station[[RES]][[chosen[is]]][ station[[RES]][[chosen[is]]]$time %in% xx$time ]
#         xx = xx[ xx$time %in% cc$time ]
#         diff = cc[[var]]-xx[[var]]
#         if(length(diff)==0) next
#         if(var=="dd"){
#           diff = diff + 360*(diff< -180) - 360*(diff> 180)
#           keep = cc$ff>.5 & xx$ff>.5
#           keep = which( keep )
#           diff = diff[ keep ]
#           xx = xx[keep]
#         }
#         winter = as.numeric(strftime(xx$time,"%m",tz="UTC")) %in% 4:9
#         summer = !winter
#         stats[is,colcount<-colcount+1] = ( mean(diff[summer]) )
#         stats[is,colcount<-colcount+1] = ( mean(diff[winter]) )
#         stats[is,colcount<-colcount+1] = ( sqrt(mean(diff[summer]^2)) )
#         stats[is,colcount<-colcount+1] = ( sqrt(mean(diff[winter]^2)) )
#         if(var=="dd"){
#           A=data$l[[ffdd]]
#           B=data$r[[ffdd]]
#           keep = !is.na(A) & !is.na(B)
#           B = B[ keep ]
#           A = A[ keep ]
#           cA = circular(A,units="degree")
#           cB = circular(B,units="degree")
#           mmdata[mmcount<-mmcount+1] = cor.circular(cA,cB)^2
#         }else{
#           A=data$l[[ffdd]]
#           B=data$r[[ffdd]]
#           keep = !is.na(A) & !is.na(B)
#           B = B[ keep ]
#           A = A[ keep ]
#           A = A  +360*((A-B)< -180)  -360*((A-B)>180)
#           mmdata[mmcount<-mmcount+1] = cor(A,B)^2
#         }
#         
#       }#var
#     }#is
#     stats$h = station$meta$height[chosen]
#     stats$hc = round(hsurf[ cbind(station$meta$ix[chosen] , station$meta$iy[chosen]) ])
#     stats$fr = ( land[ cbind(station$meta$ix[chosen] , station$meta$iy[chosen]) ] )
#     stats$src = station$meta$source[chosen]
#     
#     STATS[[RES]] = stats
#     
#   }#RES
#   save(STATS,file="T:/cosmoPaper/surface/rdata/statistic.rdata")
# }
