REMOTE=TRUE
#REMOTE=FALSE

library(data.table)

if(REMOTE){ 
  inpath="T:/cosmoPaper/surface/bas/org"
  outpath="T:/cosmoPaper/surface/bas/rdata"
}else{ 
  inpath="/home/sida6serv02/zentek/cosmoPaper/surface/bas/org"
  outpath="/home/sida6serv02/zentek/cosmoPaper/surface/bas/rdata"
}

### Surface ###
if(1){
  files = dir(paste(inpath,"/SURFACE/",sep=""))
  
  stations = list()
  for( ifile in seq(files) ){
    
    file=paste(inpath,"/SURFACE/",files[ifile],sep="")
    print(files[ifile])   
    
    formatTable = "Data format is: Year, Month, Day, Hour (UTC), Minute, Sea Level Pressure (hPa), Station Level Pressure (hPa), Temperature (Deg C), Wind Speed (knots), Wind Direction (Degrees)"
    if( read.table(file,nrows=1,skip=3,stringsAsFactors=FALSE,sep="&") != formatTable ){ print("ERROR"); break}
    meta = read.table(file,nrows=6,stringsAsFactors=FALSE,fill=TRUE)
    data = read.table(file,skip=6,stringsAsFactors=FALSE,fill=TRUE,na.strings="-999")
    data = data[which(!is.na(data$V1)),]
    
    data = data.table(
      time = as.POSIXct(paste(data$V1,"/",data$V2,"/",data$V3," ",data$V4,":",data$V5,sep=""),tz="UTC"),
      pmsl = as.numeric(data$V6),
      ps = as.numeric(data$V7),
      t = as.numeric(data$V8),
      ff = as.numeric(data$V9) * 0.514444,
      dd = as.numeric(data$V10)
    )
    
    stations[[ifile]] = list(
      name = meta[1,1],
      lon = as.numeric(meta[3,4]),
      lat = as.numeric(meta[3,2]),
      height = as.numeric(sub("m","",meta[3,6])),
      data = data
    )
    
  }
  if(stations[[1]]$name=="Amundsen_Scott"){stations[[1]]$name="Amundsen-Scott"}
  
  save(stations,file=paste(outpath,"bas_surface.rData",sep="/"))
}

### AWS ###
if(1){
  
  files = dir(paste(inpath,"/AWS/",sep=""))
  
  stations = list()
  for( ifile in seq(files) ){
    
    file=paste(inpath,"/AWS/",files[ifile],sep="")
    print(files[ifile])  
    
    formatTable = "Data format is: Year, Month, Day, Hour (UTC), Minute, Pressure (hPa), Temperature (Deg C), Wind Speed (knots), Wind Direction (Degrees)"
    if( read.table(file,nrows=1,skip=3,stringsAsFactors=FALSE,sep="&") != formatTable ){ print("ERROR"); break}
    meta = read.table(file,nrows=6,stringsAsFactors=FALSE,fill=TRUE)
    data = read.table(file,skip=6,stringsAsFactors=FALSE,fill=TRUE,na.strings="-999")
    data = data[which(!is.na(data$V1)),]
    
    data = data.table(
      time = as.POSIXct(paste(data$V1,"/",data$V2,"/",data$V3," ",data$V4,":",data$V5,sep=""),tz="UTC"),
      ps = as.numeric(data$V6),
      t = as.numeric(data$V7),
      ff = as.numeric(data$V8)* 0.514444,
      dd = as.numeric(data$V9)
    )
    
    stations[[ifile]] = list(
      name = meta[1,1],
      lon = as.numeric(meta[3,4]),
      lat = as.numeric(meta[3,2]),
      height = as.numeric(sub("m","",meta[3,6])),
      data = data
    )
    
  }
  
  save(stations,file=paste(outpath,"bas_aws.rData",sep="/"))

}

### POLENET_AWS ###
if(1){
  
  files = dir(paste(inpath,"/POLENET_AWS/",sep=""))
  
  stations = list()
  for( ifile in seq(files) ){
    
    file=paste(inpath,"/POLENET_AWS/",files[ifile],sep="")
    print(files[ifile])  
    
    formatTable = "Data format is: Year, Month, Day, Hour (UTC), Minute, Pressure (hPa), Temperature (Deg C), Wind Speed (knots), Wind Direction (Degrees)"
    if( read.table(file,nrows=1,skip=3,stringsAsFactors=FALSE,sep="&") != formatTable ){ print("ERROR"); break}
    meta = read.table(file,nrows=6,stringsAsFactors=FALSE,fill=TRUE)
    data = read.table(file,skip=6,stringsAsFactors=FALSE,fill=TRUE,na.strings="-999")
    data = data[which(!is.na(data$V1)),]
    
    data = data.table(
      time = as.POSIXct(paste(data$V1,"/",data$V2,"/",data$V3," ",data$V4,":",data$V5,sep=""),tz="UTC"),
      ps = as.numeric(data$V6),
      t = as.numeric(data$V7),
      ff = as.numeric(data$V8)* 0.514444,
      dd = as.numeric(data$V9)
    )
    
    stations[[ifile]] = list(
      name = meta[1,1],
      lon = as.numeric(meta[3,4]),
      lat = as.numeric(meta[3,2]),
      height = as.numeric(sub("m","",meta[3,6])),
      data = data
    )
    
  }
  
  save(stations,file=paste(outpath,"bas_polenetaws.rData",sep="/"))
  
}

for( i in seq(stations) ){
  print(stations[[i]]$data$time[1:3])
}
