library(ncdf4)
library(data.table)

# Init
if(0){
  load(file="T:/cosmoPaper/surface/rdata/stationFinal.rData")
  source("T:/cosmoPaper/surface/strip_metvurst.R")
  
  RES="C15"
  file = paste("S:/COSMO_",RES,"_out01/20040404/out01/lffd2004040318c.nc",sep="")
  nc = nc_open(file)
  land = ncvar_get(nc,"FR_LAND")
  rlon = ncvar_get(nc,"rlon")
  rlat = ncvar_get(nc,"rlat")
  nc_close(nc)

}

# south # east coast # south Peninsula # middle peninsula # north peninsula
chosen = c(5,12 , 47,65,107 , 33,45,30 , 41,43,50,53 , 76,78,81,83,87,89)

### own plot of chosen
if(0){
  PRINT = 1
  everyTIME = 1
  for( i in seq(chosen)[] ){
    if(PRINT) png(paste("T:/cosmoPaper/surface/pics/",i,"_",station$meta$name[chosen[i]],"_own.png",sep=""),height=1000,width=1600,res=100)
    par(mfrow=c(3,4),omi=c(.3,.1,.1,.1),mai=rep(0,4))
    x = station$data[[chosen[i]]]
    m = station$meta[chosen[i]]
    index = seq(1,length(x$time),everyTIME)
    XLIM = as.POSIXct(c("2002/01/01","2019/01/01"),tz="UTC")
    TAXIS = as.POSIXct(c("2005/01/01","2010/01/01","2015/01/01"),tz="UTC")
    
    ### map ###
    image(rlon,rlat,land,col=c("cornflowerblue","gray"),asp=1,xaxt="n",yaxt="n")
    points(m$rlon,m$rlat,col=2,pch=4,cex=2,lwd=2)
    points(station$meta$rlon,station$meta$rlat,pch=20)

    ### meta ###
    plot(0,0,col=0,xaxt="n",yaxt="n")
    text(-1,0,paste(paste(names(m),": ",m,sep=""),collapse="\n"),font=2,cex=2,pos=4)
    
    ### year time overview ###
    doy = as.numeric(strftime(x$time[index],"%j"))
    year = as.numeric(strftime(x$time[index],"%Y")) -2000
    plot(0,0,col=0,xlim=c(0,400),ylim=c(1,18),xaxt="n",yaxt="n")
    abline(h=2:17,col="gray")
    points(doy,year,pch=20)
    text(180,17,"availible data",cex=2)
    text(rep(390,2),c(5,10,15),c(2005,2010,2015))
    
    ### hours ###
    plot(0,0,col=0,xlim=XLIM,ylim=c(-2,25),xaxt="n",yaxt="n")
    abline(h=seq(0,24,6),col="gray")
    diff = diff(as.numeric(x$time)/3600)
    points(x$time[-1],diff,pch=20)
    text(XLIM[2],c(seq(0,18,6),25),c(seq(0,18,6),">24"))
    text(mean(par("usr")[1:2]),19,paste("data every # hour(s)\nmean =",round(mean(diff(as.numeric(x$time)))/3600),"\nmedian =",round(median(diff(as.numeric(x$time)))/3600)),cex=2)
    text(TAXIS,-1,as.numeric(strftime(TAXIS,"%Y")),font=2,cex=1.5,col="cornflowerblue")
    
    ### vars ###
    YLIM = list(t=c(-80,30),ff=c(0,30),dd=c(0,360),ps=c(600,1050),pmsl=c(600,1050))
    if(m$source=="noaa"){ VAR = c("t","ff","dd","pmsl") }
    else if(grep("bas",m$source)){ VAR = c("t","ff","dd","ps") }
    for(var in VAR ){
      YYLIM = YLIM[[var]]-c(0.05*diff(YLIM[[var]]),0)
      plot(0,0,col=0,xlim=XLIM,ylim=YYLIM,xaxt="n",yaxt="n")
      # watersign
      text(mean(XLIM),mean(YLIM[[var]]),var,font=2,cex=10,col="gray")
      # y axis
      AT = seq(YLIM[[var]][1],YLIM[[var]][2],10*round(diff(YLIM[[var]])/30))
      abline( h=AT,col="gray" )
      text(XLIM[2],AT,AT)
      # x axis
      text(TAXIS,YYLIM[1],as.numeric(strftime(TAXIS,"%Y")),font=2,cex=1.5,col="cornflowerblue")
      # plot cclm
      cCol=list(C15="dodgerblue",T15="forestgreen")
      cindex=seq(1,length(station[[RES]][[chosen[i]]]$time),everyTIME)
      for( RES in c("C15","T15") ){
        points( station[[RES]][[chosen[i]]]$time[cindex], station[[RES]][[chosen[i]]][[var]][cindex], col=cCol[[RES]], pch=19)
      }
      # plot meassurement
      points(x$time[index],x[[var]][index],pch=20)
      # plot missing
      missing = ( is.na(x[[var]]) | is.nan(x[[var]]) | YLIM[[var]][1]>x[[var]] | x[[var]]>YLIM[[var]][2] )
      points(x$time[missing],rep(YYLIM[1],sum(missing)),col="firebrick",pch=20)
      text(mean(par("usr")[1:2]),YLIM[[var]][2],paste("NA =",round(sum(missing)/length(missing)*100),"%"),col="firebrick",cex=2)
    }
    
    ### hist hours ###
    hist(as.numeric(strftime(x$time[index],"%H")),breaks=-0.5:23.5,xlim=c(-0.5,23.5),yaxt="n",main="")
    text(mean(par("usr")[1:2]),mean(par("usr")[3:4]),"hours",cex=2)

    ### hist month ###
    hist(as.numeric(strftime(x$time[index],"%m")),breaks=0.5:12.5,xlim=c(0.5,12.5),yaxt="n",main="")
    text(mean(par("usr")[1:2]),mean(par("usr")[3:4]),"month",cex=2)
    
    ### hist years ###
    hist(as.numeric(strftime(x$time[index],"%Y")),breaks=2001.5:2018.5,xlim=c(2001.5,2018.5),yaxt="n",main="")
    text(mean(par("usr")[1:2]),mean(par("usr")[3:4]),"years",cex=2)
    
    ### meta ###
    plot(0,0,col=0,xaxt="n",yaxt="n")
    text(0,0,i,font=2,cex=10)
    
    if(PRINT) dev.off()
  }
}

### strip_metvurst
if(0){
  PRINT = 1
  for( i in seq(chosen) ){
    is=chosen[i]
    m = station$meta[is]
    x = station$data[[is]]
    
    ### vars ###
    if(m$source=="noaa"){ VAR = c("t","ff","dd","pmsl") }
    else if(grep("bas",m$source)){ VAR = c("t","ff","dd","ps")[2:3] }
    for(var in VAR ){
      for( RES in c("C15","T15")[1] ){
        xx = x[ !is.na(x[[var]]) ]
        double = which(duplicated(xx$time))
        if(length(double)>0){ xx = xx[ -which(duplicated(xx$time)) ] }
        cc = station[[RES]][[is]][ station[[RES]][[is]]$time %in% xx$time ]
        xx = xx[ xx$time %in% cc$time ]
        diff = cc[[var]]-xx[[var]]
        if(length(diff)==0) next
        if(var=="dd"){
          diff = diff + 360*(diff< -180) - 360*(diff> 180)
          keep = cc$ff>.5 & xx$ff>.5
          keep = which( keep )
          diff = diff[ keep ]
          xx = xx[keep]
        }
        date = strftime(xx[["time"]],"%Y-%m-%d %H:%M:%S",tz="UTC")
        cond = strftime(xx[["time"]],"%Y",tz="UTC")
        range = list(t=c(-20,20),ff=c(-20,20),dd=c(-90,90),ps=c(-20,20),pmsl=c(-20,20))
        diff[ diff > range[[var]][2] ] = range[[var]][2]
        diff[ diff < range[[var]][1] ] = range[[var]][1]
        plot.temp = strip_metvurst(x=diff,date=date,cond=cond,range=range[[var]],arrange="long",main=paste(m$name,var,RES))
        if(PRINT) png(paste("T:/cosmoPaper/surface/pics/",i,"_",station$meta$name[chosen[i]],"_",var,"_",RES,".png",sep=""),height=1000,width=1600,res=100)
        print(plot.temp)
        if(PRINT) dev.off()
      }#RES
    }#var
    ### meta ###
    

  }
}

### Chosen
if(0){
  par(mfrow=c(1,1),omi=c(.1,.1,.1,.1),mai=rep(0,4))
  
  ### map ###
  image(rlon,rlat,land,col=c("cornflowerblue","gray"),asp=1,xaxt="n",yaxt="n")
  points(station$meta$rlon[good],station$meta$rlat[good],pch=20)
  points(station$meta$rlon[chosen],station$meta$rlat[chosen],pch=4,lwd=5,cex=2,col="firebrick")
  
}

### check for 78 - 12 h timeshift values #wrote steve and got corrected
if(0){

  # par(mfrow=c(5,4),omi=c(0,0,0,0),mai=c(.3,.4,.2,0))
  # for( i in chosen){
  #   hist( diff(as.numeric(station$data[[i]]$time))/60 ,xlim=c(0,400),breaks = c(-10^100,0:400,10^100), main=print(station$meta$name[i]) )
  # }
  par(mfrow=c(1,1),omi=c(0,0,0,0),mai=c(.9,1.3,.5,.5),cex=1.5)
  XLIM = as.POSIXct(c("2009/04/01","2009/04/30"),tz="UTC")
  YLIM = c(-10,5)
  COL = c("red","purple","orange","black","dodgerblue")
  LWD = c(2,2,2,5,5)
  plot(XLIM,YLIM,col=0,ylim=YLIM,xlim=XLIM,main="2009",ylab="Temperature (C)")
  grid()
  IS = 78
  lines( station$data[[IS]]$time, station$data[[IS]]$t,type="l",col=COL[4],lwd=LWD[4])
  lines( station$data[[IS]]$time+3600*12, station$data[[IS]]$t,type="l",col=COL[5],lwd=LWD[5])
  lines( station$C15[[IS]]$time, station$C15[[IS]]$t , col=COL[1],lwd=LWD[1])
  IS = 83
  lines( station$data[[IS]]$time, station$data[[IS]]$t,col=COL[2],lwd=LWD[2] )
  lines( station$C15[[IS]]$time, station$C15[[IS]]$t , col=COL[1],lwd=LWD[1])
  IS = 81
  lines( station$data[[IS]]$time, station$data[[IS]]$t,col=COL[3],lwd=LWD[3] )
  lines( station$C15[[IS]]$time, station$C15[[IS]]$t , col=COL[1],lwd=LWD[1])

  legend("bottomleft",c("simulation","Bellingshausen","Marsh","Great_Wall original","Great_Wall newTime = oldTime+12h"),col=COL,lwd=LWD)
}

### overview all Chosen
if(0){
  PRINT = 1
  everyTIME = 1000
  for( is in chosen ){
    if(PRINT) png(paste("T:/scr/surface/pics/chosen/",is,"s.png",sep=""),height=1000,width=1600,res=100)
    par(mfrow=c(4,1),omi=c(.3,.1,.1,.1),mai=rep(0,4))
    x = station$data[[is]]
    m = station$meta[is]
    index = seq(1,length(x$time),everyTIME)
    XLIM = as.POSIXct(c("2002/01/01","2019/01/01"),tz="UTC")
    TAXIS = as.POSIXct(c("2005/01/01","2010/01/01","2015/01/01"),tz="UTC")
    
    ### vars ###
    stats = station$meta$name[is]
    YLIM = list(t=c(-10,20),ff=c(-10,20),dd=c(-60,120),ps=c(-50,100),pmsl=c(-50,100))
    if(m$source=="noaa"){ VAR = c("t","ff","dd","pmsl") }
    else if(grep("bas",m$source)){ VAR = c("t","ff","dd","ps") }
    for(var in VAR ){
      YYLIM = YLIM[[var]]-c(0.05*diff(YLIM[[var]]),0)
      plot(0,0,col=0,xlim=XLIM,ylim=YYLIM,xaxt="n",yaxt="n")
      # watersign
      text(mean(XLIM),mean(YLIM[[var]]),var,font=2,cex=15,col="gray")
      # y axis
      AT = seq(YLIM[[var]][1],YLIM[[var]][2],10*round(diff(YLIM[[var]])/30))
      abline( h=AT,col="gray" )
      text(XLIM[2],AT,AT)
      # x axis
      text(TAXIS,YYLIM[1],as.numeric(strftime(TAXIS,"%Y")),font=2,cex=1.5,col="cornflowerblue")
      # plot cclm
      cCol=list(C15="dodgerblue",T15="forestgreen")
      yPos=list(C15=c(.75,.25),T15=c(.25,.75))
      for( RES in c("C15","T15") ){
        xx = x[ !is.na(x[[var]]) ]
        double = which(duplicated(xx$time))
        if(length(double)>0){ xx = xx[ -which(duplicated(xx$time)) ] }
        cc = station[[RES]][[is]][ station[[RES]][[is]]$time %in% xx$time ]
        xx = xx[ xx$time %in% cc$time ]
        diff = cc[[var]]-xx[[var]]
        if(length(diff)==0) next
        if(var=="dd"){
          diff = diff + 360*(diff< -180) - 360*(diff> 180)
          keep = cc$ff>.5 & xx$ff>.5
          keep = which( keep )
          diff = diff[ keep ]
          xx = xx[keep]
        }
        bias= mean(diff)
        rmse = sqrt(mean(diff^2))
        # plot stats
        lines(xx$time[index], runmed(diff,5)[index], col=cCol[[RES]])
        text((par("usr")[1:2])%*%c(.08,.92),(par("usr")[3:4])%*%yPos[[RES]],paste("bias =",round(bias,2),"\nrmse = ",round(rmse,2)),col=cCol[[RES]],cex=2)
        stats = c(stats,round(bias,1))
      }#RES
    }#var
    print( paste(c(sprintf("%35s",stats[1]),sprintf(" | %5s",stats[-1])),collapse="") )
    ### meta ###
    text(mean(par("usr")[1:2]),mean(par("usr")[3:4]),paste(i,m$name),font=2,cex=10)
    
    if(PRINT) dev.off()
  }
}
